<?php

/**
 *	Class AuthorsAccounts
 *  -------------- 
 *  Description : encapsulates Authors Accounts operations & properties
 *  Updated	    : 25.04.2010
 *	Written by  : ApPHP
 *	
 **/

class AuthorsAccounts extends MicroGrid {
	
	protected $debug = false;

	protected $tableName;
	protected $primaryKey;
	protected $dataSet;
	protected $formActionURL;
	protected $params;
	protected $actions;

	protected $languageId;
	protected $allowLanguages;

	protected $VIEW_MODE_SQL;
	protected $EDIT_MODE_SQL;
	protected $DETAILS_MODE_SQL;
	protected $WHERE_CLAUSE;
	protected $ORDER_CLAUSE;
	protected $LIMIT_CLAUSE;

	protected $arrViewModeFields;	
	protected $arrAddModeFields;	
	protected $arrEditModeFields;
	protected $arrDetailsModeFields;
	
	protected $isPagingAllowed;
	protected $pageSize;

	protected $isSortingAllowed;

	protected $isFilteringAllowed;

	public $error;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct($account_type = "", $logged_user_type = "author"){

		$this->SetRunningTime();
		
		$this->params = array();		
		if(isset($_POST['first_name'])) 	$this->params['first_name']  	= $_POST['first_name'];
		if(isset($_POST['last_name']))		$this->params['last_name']  	= $_POST['last_name'];
		if(isset($_POST['user_name']))  	$this->params['user_name']  	= $_POST['user_name'];
		if(isset($_POST['password']))		$this->params['password']   	= $_POST['password'];
		if(isset($_POST['email']))   		$this->params['email']      	= $_POST['email'];
		if(isset($_POST['account_type'])) 	$this->params['account_type'] 	= $_POST['account_type'];
		if(isset($_POST['is_active']))  	$this->params['is_active']  	= $_POST['is_active']; else $this->params['is_active'] = "0";
		if(isset($_POST['about_me']))   	$this->params['about_me']       = $_POST['about_me'];
		$this->params['show_about_me']  =  (isset($_POST['show_about_me'])) ? $_POST['show_about_me'] : "0";

		## for images
		if(isset($_POST['avatar'])){
			$this->params['avatar'] = $_POST['avatar'];
		}else if(isset($_FILES['avatar']['name']) && $_FILES['avatar']['name'] != ""){
			// nothing 			
		}else if (self::GetParameter('action') == "create"){
			$this->params['avatar'] = "";
		}
		
		$this->primaryKey 	= "id";
		$this->tableName 	= DB_PREFIX."accounts";	
		$this->dataSet 		= array();
		$this->error 		= "";
		///$this->languageId  	= (isset($_REQUEST['language_id']) && $_REQUEST['language_id'] != "") ? $_REQUEST['language_id'] : Languages::GetDefaultLang();
		
		if($account_type == "me"){
			$this->formActionURL = "index.php?admin=my_account";	
		}else{
			$this->formActionURL = "index.php?admin=authors_management";	
		}		
		$this->actions      = array("add"=>true, "edit"=>true, "details"=>true, "delete"=>true);
		$this->actionIcons  = true;

		$this->allowLanguages = false;
		$this->WHERE_CLAUSE = "WHERE ".DB_PREFIX."accounts.account_type = 'author' ";		
		$this->ORDER_CLAUSE = "ORDER BY id ASC";

		$this->isAlterColorsAllowed = true;

		$this->isPagingAllowed = true;
		$this->pageSize = 10;

		$this->isSortingAllowed = true;
		
		$this->isFilteringAllowed = true;
		// define filtering fields
		$this->arrFilteringFields = array(
			_FIRST_NAME => array("table"=>$this->tableName, "field"=>"first_name", "type"=>"text", "sign"=>"like%", "width"=>"80px"),
			_LAST_NAME  => array("table"=>$this->tableName, "field"=>"last_name", "type"=>"text", "sign"=>"like%", "width"=>"80px"),
		);

		// prepare languages array		
		#$total_languages = Languages::GetAll();
		#$arr_languages      = array();
		#foreach($total_languages[0] as $key => $val){
		#	$arr_languages[$val['abbreviation']] = $val['lang_name'];
		#}

		//---------------------------------------------------------------------- 
		// VIEW MODE
		//---------------------------------------------------------------------- 
		$this->VIEW_MODE_SQL = "SELECT ".$this->primaryKey.",
									first_name,
		                            last_name,
									user_name,
									email,
									account_type,
									last_login,
									about_me,
									show_about_me,
									IF(is_active, '<font color=\"00a600\">"._YES."</font>', '<font color=\"a60000\">"._NO."</font>') as is_active
								FROM ".$this->tableName."";		
		// define view mode fields
		$this->arrViewModeFields = array(
			"first_name" => array("title"=>_FIRST_NAME,  "type"=>"label", "align"=>"left", "width"=>""),
			"last_name"  => array("title"=>_LAST_NAME,  "type"=>"label", "align"=>"left", "width"=>""),
			"user_name"  => array("title"=>_USER_NAME,  "type"=>"label", "align"=>"left", "width"=>""),
			"email"      => array("title"=>_EMAIL_ADDRESS, "type"=>"label", "align"=>"left", "width"=>""),
			"last_login" => array("title"=>_LAST_LOGIN, "type"=>"label", "align"=>"center", "width"=>""),
			"is_active"  => array("title"=>_ACTIVE, "type"=>"label", "align"=>"center", "width"=>""),
		);
		
		//---------------------------------------------------------------------- 
		// ADD MODE
		//---------------------------------------------------------------------- 
		// define add mode fields
		$this->arrAddModeFields = array(
			"separator_1"   =>array(
				"separator_info" => array("legend"=>_PERSONAL_INFO),
				"first_name"  	=> array("title"=>_FIRST_NAME,	"type"=>"textbox", "width"=>"210px", "required"=>true, "validation_type"=>"text"),
				"last_name"    	=> array("title"=>_LAST_NAME, 	"type"=>"textbox", "width"=>"210px", "required"=>true, "validation_type"=>"text"),
				"email" 		=> array("title"=>_EMAIL_ADDRESS,"type"=>"textbox", "width"=>"210px", "required"=>true, "validation_type"=>"email"),
				"user_name"  	=> array("title"=>_USER_NAME,	"type"=>"textbox", "width"=>"210px", "required"=>true, "validation_type"=>"alpha_numeric"),
				"password"  	=> array("title"=>_PASSWORD, 	"type"=>"password", "width"=>"210px", "required"=>true, "validation_type"=>"password", "cryptography"=>PASSWORDS_ENCRYPTION, "cryptography_type"=>PASSWORDS_ENCRYPTION_TYPE, "aes_password"=>PASSWORDS_ENCRYPT_KEY),
			),
			"separator_2"   =>array(
				"separator_info" => array("legend"=>_ABOUT_ME_BLOCK),
				"avatar"        => array("title"=>_AVATAR." (32x32)",      "type"=>"image",    "width"=>"210px", "required"=>false, "readonly"=>false, "target"=>"images/users_images/", "random_name"=>"true"),
				"about_me"      => array("title"=>_ABOUT_ME, "type"=>"textarea", "width"=>"310px", "height"=>"90px", "required"=>false, "readonly"=>false, "default"=>"", "validation_type"=>""),
			),
			"separator_3"   =>array(
				"separator_info" => array("legend"=>_ABOUT_ME_BLOCK),
				"account_type" 	=> array("title"=>"",           "type"=>"hidden",  "required"=>false, "default"=>"author"),
				//"account_type"  => array("title"=>_ACCOUNT_TYPE, "type"=>"enum", "required"=>true, "readonly"=>false, "width"=>"210px", "source"=>array("author"=>"Author")),
				"last_login" 	=> array("title"=>"",           "type"=>"hidden",  "required"=>false, "default"=>""),
				"is_active"  	=> array("title"=>_ACTIVE,	    "type"=>"checkbox", "true_value"=>"1", "false_value"=>"0"),
			),			
		);

		if($account_type == ""){
			$this->arrAddModeFields["separator_2"]["show_about_me"] = array("title"=>_SHOW_ABOUT_ME, "type"=>"hidden", "required"=>false, "default"=>"0");
		}

		//---------------------------------------------------------------------- 
		// EDIT MODE
		//---------------------------------------------------------------------- 
		$this->EDIT_MODE_SQL = "SELECT
								".$this->tableName.".".$this->primaryKey.",
								".$this->tableName.".first_name,
								".$this->tableName.".last_name,
								".$this->tableName.".user_name,
								".$this->tableName.".password,
								".$this->tableName.".email,
								".$this->tableName.".avatar,
								".$this->tableName.".about_me,
								".$this->tableName.".show_about_me,
								IF(show_about_me, 'Yes', 'No') as my_show_about_me,
								".$this->tableName.".account_type,
								IF(account_type = 'admin', '"._ADMINS."', '"._AUTHORS."') as my_account_type,
								".$this->tableName.".last_login,
								".$this->tableName.".is_active
							FROM ".$this->tableName."
							WHERE ".$this->tableName.".".$this->primaryKey." = _RID_";		
		// define edit mode fields
		$this->arrEditModeFields = array(
			"separator_1"   =>array(
				"separator_info" => array("legend"=>_PERSONAL_INFO),
				"user_name"  	=> array("title"=>_USER_NAME,	"type"=>"label"),
				"first_name"  	=> array("title"=>_FIRST_NAME,	"type"=>"textbox", "width"=>"210px", "required"=>true, "validation_type"=>"text"),
				"last_name"    	=> array("title"=>_LAST_NAME, 	"type"=>"textbox", "width"=>"210px", "required"=>true, "validation_type"=>"text"),
				"email" 		=> array("title"=>_EMAIL_ADDRESS,"type"=>"textbox", "width"=>"210px", "required"=>true, "validation_type"=>"email"),
				//"password"  	=> array("title"=>_PASSWORD, 	"type"=>"password", "width"=>"210px", "required"=>true, "validation_type"=>"password", "cryptography"=>PASSWORDS_ENCRYPTION, "cryptography_type"=>PASSWORDS_ENCRYPTION_TYPE, "aes_password"=>PASSWORDS_ENCRYPT_KEY),
			),
			"separator_2"   =>array(
				"separator_info" => array("legend"=>_ABOUT_ME_BLOCK),
				"avatar"        => array("title"=>_AVATAR." (32x32)", "type"=>"image",    "width"=>"210px", "required"=>false, "readonly"=>false, "target"=>"images/users_images/", "random_name"=>"true"),
				"about_me"      => array("title"=>_ABOUT_ME, "type"=>"textarea", "width"=>"310px", "height"=>"90px", "required"=>false, "readonly"=>false, "default"=>"", "validation_type"=>""),
			),
			"separator_3"   =>array(
				"separator_info" => array("legend"=>_OTHER),
				"my_account_type"   => array("title"=>_ACCOUNT_TYPE, "type"=>"label"),
				"last_login"        => array("title"=>"", "type"=>"hidden", "required"=>false, "default"=>""),				
			)
		);
		
		if($account_type == ""){
			$this->arrEditModeFields["separator_2"]["show_about_me"] = array("title"=>_SHOW_ABOUT_ME, "type"=>"hidden", "required"=>false, "default"=>"0");
			$this->arrEditModeFields["separator_3"]["is_active"] = array("title"=>_ACTIVE, "type"=>"checkbox", "true_value"=>"1", "false_value"=>"0");
		}else{
			// My account page
			if($logged_user_type == "admin") $this->arrEditModeFields["separator_2"]["show_about_me"] = array("title"=>_SHOW_ABOUT_ME, "type"=>"checkbox", "true_value"=>"1", "false_value"=>"0");
			$this->arrEditModeFields["separator_3"]["is_active"] = array("title"=>_ACTIVE, "type"=>"hidden", "required"=>false, "default"=>"1");
		}

		//---------------------------------------------------------------------- 
		// DETAILS MODE
		//----------------------------------------------------------------------
		$this->DETAILS_MODE_SQL = "SELECT
								".$this->tableName.".".$this->primaryKey.",
								".$this->tableName.".first_name,
								".$this->tableName.".last_name,
								".$this->tableName.".user_name,
								".$this->tableName.".password,
								".$this->tableName.".email,
								".$this->tableName.".avatar,
								".$this->tableName.".about_me,
								".$this->tableName.".show_about_me,
								IF(show_about_me, 'Yes', 'No') as my_show_about_me,
								IF(account_type = 'admin', '"._ADMINS."', '"._AUTHORS."') as my_account_type,
								".$this->tableName.".last_login,
								IF(is_active, 'Yes', 'No') as is_active
							FROM ".$this->tableName."
							WHERE ".$this->tableName.".".$this->primaryKey." = _RID_";		
		$this->arrDetailsModeFields = array(
			"separator_1"   =>array(
				"separator_info" => array("legend"=>_PERSONAL_INFO),
				"user_name"   	=> array("title"=>_USER_NAME, "type"=>"label"),
				"first_name"  	=> array("title"=>_FIRST_NAME,	"type"=>"label"),
				"last_name"    	=> array("title"=>_LAST_NAME, "type"=>"label"),
				"email"     	=> array("title"=>_EMAIL_ADDRESS, 	 "type"=>"label"),				
			),
			"separator_2"   =>array(
				"separator_info" => array("legend"=>_ABOUT_ME_BLOCK),
				"avatar"        => array("title"=>_AVATAR, "type"=>"image", "target"=>"images/users_images/", "no_image"=>"no_image.png"),			
				"about_me"      => array("title"=>_ABOUT_ME, "type"=>"label"),
			),
			"separator_3"   =>array(
				"separator_info" => array("legend"=>_OTHER),
				"my_account_type"  => array("title"=>_ACCOUNT_TYPE, "type"=>"label"),
				"last_login" 	=> array("title"=>_LAST_LOGIN, "type"=>"label"),
				"is_active"  	=> array("title"=>_ACTIVE, "type"=>"label"),
			)
		);

		if($account_type == "me"){
			$this->arrDetailsModeFields["separator_2"]["my_show_about_me"] = array("title"=>_SHOW_ABOUT_ME, "type"=>"label");
		}

	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct()
	{
		// echo 'this object has been destroyed';
    }

	function PostAddRecord(){
		global $objSettings;

		////////////////////////////////////////////////////////////
		$sender = $objSettings->GetParameter("admin_email");
		$recipiant = $this->params['email'];
		
		$msg_title = _ACCOUNT_WAS_CREATED;        
		$message = file_get_contents("mails/account_created.txt");
		$message = str_replace("_FIRST_NAME_", $this->params['first_name'], $message);
		$message = str_replace("_LAST_NAME_", $this->params['last_name'], $message);
		$message = str_replace("_USER_NAME_", $this->params['user_name'], $message);
		$message = str_replace("_USER_PASSWORD_", $this->params['password'], $message);
		$message = str_replace("_SITE_NAME_", $objSettings->GetParameter("header_text"), $message);
		$message = str_replace("_BASE_URL_", get_base_url(), $message);				
		
		$textversion = strip_tags($message);
		$htmlversion = $message;
	
		unset($msg);	  
		$msg = new Email($recipiant, $sender, $msg_title); 
		
		$msg->textOnly = false;
		$msg->content = $htmlversion;
	
		$sendsuccess = $msg->Send();
		////////////////////////////////////////////////////////////
	}

	/***
	 *	Get all DataSet array
	 *
	 *  static public function GetAll() { }
	 *	
	 **/
	
	/***
	 *	Add record
	 *
	 *	function AddRecord($params = array()) { 	}
	 *	
	 **/

	/***
	 *	Delete record
	 *
	 *	function DeleteRecord($rid = "") { 	}
	 *	
	 **/

}
?>