<?php

/**
 *	Class Category
 *  -------------- 
 *  Description : encapsulates category properties
 *  Updated	    : 16.05.2009
 *	Written by  : ApPHP
 *	
 **/

class Categories {

	private $id;
	
	protected $category;
	protected $whereClause;
	
	public $error;    
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct($id = "") {
		$this->id = $id;
		$this->whereClause  = "";
		
		if($this->id!=""){
			$sql = "SELECT
						".DB_PREFIX."categories.*
					FROM ".DB_PREFIX."categories
					WHERE ".DB_PREFIX."categories.id = '".intval($this->id)."'";
			$this->category = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);
		}else{
			$this->category['category_name'] = "";
			$this->category['category_order'] = "";
		}
	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct(){
		// echo 'this object has been destroyed';
    }

	/***
	 *	Get category name
	 *	
	 **/
	public function GetName(){		
		if(isset($this->category['category_name'])) return decode_text($this->category['category_name']);
		else return "";
	}

	/***
	 *	Get parameter
	 *	
	 **/
	public function GetParameter($param = ""){
		if(isset($this->category[$param])){
			return $this->category[$param];
		}else{
			return "";
		}
	}
	
	/***
	 *	Get category ID
	 *	
	 **/
	public function GetId(){
		return $this->id;
	}
	
	/***
	 *	Get category order
	 *	
	 **/
	public function GetOrder(){
		if(isset($this->category['category_order'])) return $this->category['category_order'];
		else return "";
	}
	
	/***
	 *	Get all top categories array
	 *	
	 **/
	function GetAllTop($where_clause = ""){
		
		// Get all top categories
		$sql = "SELECT ".DB_PREFIX."posts.* 
				FROM ".DB_PREFIX."posts
				WHERE 1=1 AND ".DB_PREFIX."posts.is_published = 1 ".$where_clause;
		return database_query($sql, DATA_AND_ROWS);
	}

	/***
	 *	Get all left categories array
	 *	
	 **/
	function GetLeftCategories(){
		// Get all left categories
		$sql = "SELECT
					".DB_PREFIX."categories.*
				FROM ".DB_PREFIX."categories
				ORDER BY ".DB_PREFIX."categories.category_order ASC ";
		$result = database_query($sql, DATA_AND_ROWS);
		// add not-defined categories 
		$result[0][] = array("id" => "0", "category_name" => _OTHER, "category_order" => "999");
		$result[1]++;
		return $result;
	}
	
	/***
	 *	Get all left category links array
	 *	
	 **/
	function GetLeftCategoryLinks($category_id){
		// Get all left categories
		$sql = "SELECT
					".DB_PREFIX."posts.*
				FROM ".DB_PREFIX."posts
				WHERE
					".DB_PREFIX."posts.is_published = 1 AND 
					".DB_PREFIX."posts.category_id = '".$category_id."'
				ORDER BY ".DB_PREFIX."posts.id ASC ";
		return database_query($sql, DATA_AND_ROWS);
	}

	/***
	 *	Get all active categories array
	 **/
	function GetAllActive(){
		$sql = "SELECT *
				FROM ".DB_PREFIX."categories 
				WHERE 1=1
				ORDER BY category_order ASC";					
		return database_query($sql, DATA_AND_ROWS, ALL_ROWS, FETCH_ASSOC);		
	}

	/***
	 *	Get all categories array
	 *		@param $order - order clause
	 *		@param $join_table - join tables
	 **/
	function GetAll($order = " category_order ASC", $join_table = "", $post_placement = "") {
		$where_clause = "";
		if($post_placement == "left"){
			$where_clause .= "AND ".DB_PREFIX."posts.post_placement = 'left' ";
		}else if($post_placement == "top"){
			$where_clause .= "AND ".DB_PREFIX."posts.post_placement = 'top' ";
		}
		
		// Build ORDER BY CLAUSE
		if($order=="")$order_clause = "";
		else $order_clause = "ORDER BY $order";		

		// Build JOIN clause
		$join_select_fields = "";
		$join_clause = "";
		if($join_table != "categories") {
			$join_clause = "LEFT OUTER JOIN ".DB_PREFIX.$join_table." ON ".DB_PREFIX.$join_table.".category_id=".DB_PREFIX."categories.id ";
			$join_select_fields = ", ".DB_PREFIX.$join_table.".* ";
		}
		
		$sql = "SELECT
					".DB_PREFIX."categories.*
					$join_select_fields 
				FROM ".DB_PREFIX."categories 
					$join_clause
				WHERE 1=1
				$where_clause
				$order_clause";			
		
		return database_query($sql, DATA_AND_ROWS, ALL_ROWS, FETCH_ASSOC);
	}
	
	
	/***
	 *	Update category 
	 *		@param $param - array of parameters
	 **/
	function CategoryUpdate($params = array()) {

		// Block operation in demo mode
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;
		}

		if(isset($this->id)){
			// Get input parameters
			if(isset($params['name']) && $params['name']!=""){
				$this->category['category_name'] = $params['name'];
			}else{
				$this->error = _CATEGORY_NAME_EMPTY;
				return false;
			}
			if(isset($params['order'])) $this->category['category_order'] = $params['order'];
			
			$sql = "SELECT MIN(category_order) as min_order, MAX(category_order) as max_order FROM ".DB_PREFIX."categories";
			if($category = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY)){
				$min_order = $category['min_order'];
				$max_order = $category['max_order'];
				
				// insert category with new order priority in categories list
				$sql = "SELECT category_order FROM ".DB_PREFIX."categories WHERE id = ".(int)$this->id;
				if($category = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY)){
					$sql_down = "UPDATE ".DB_PREFIX."categories SET category_order = category_order - 1 WHERE id <> ".(int)$this->id." AND category_order <= ".$this->category['category_order']." AND category_order > ".$category['category_order'];
					$sql_up = "UPDATE ".DB_PREFIX."categories SET category_order = category_order + 1 WHERE id <> ".(int)$this->id." AND category_order >= ".$this->category['category_order']." AND category_order < ".$category['category_order'];
					
					if($category['category_order'] != $this->category['category_order']){							
						$sql = "UPDATE ".DB_PREFIX."categories
						        SET
									category_name = '".encode_text($this->category['category_name'])."',								
								    category_order = ".$this->category['category_order']."
								WHERE id = ".(int)$this->id." AND category_order <> ".$this->category['category_order'];
						if($result = database_void_query($sql)){
							if($this->category['category_order'] == $min_order){
								$sql = $sql_up;
							}else if($this->category['category_order'] == $max_order){
								$sql = $sql_down;
							}else{
								if($category['category_order'] < $this->category['category_order']) $sql = $sql_down;
								else $sql = $sql_up;
							}
							$result = database_void_query($sql);
						}							
					}else{
						$sql = "UPDATE ".DB_PREFIX."categories
						        SET
								    category_name = '".encode_text($this->category['category_name'])."'
								WHERE id = ".(int)$this->id;
						$result = database_void_query($sql);
					}
				}
			}

			if($result >= 0){
				return true;
			}else{
				$this->error = TRY_LATER;
				return false;
			}				
		}else{
			$this->error = CATEGORY_MISSED;
			return false;
		}
	}
	
	/***
	 *	Create new category 
	 *		@param $param - array of parameters
	 **/
	function CategoryCreate($params = array()) {

		// Block operation in demo mode
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;
		}

		// Get input parameters
		if(isset($params['name'])) $this->category['category_name'] = $params['name'];
		if(isset($params['order'])) $this->category['category_order'] = $params['order'];

		// Prevent creating of empty records in our 'categories' table
		if($this->category['category_name']!=""){

			$m = self::GetAll(" category_order ASC", "categories", "");
			$max_order = (int)($m[1]+1);			

			$sql = "INSERT INTO ".DB_PREFIX."categories (category_name, category_order)
					VALUES('".encode_text($this->category['category_name'])."', ".$max_order.")";
			if(!database_void_query($sql)){
				$this->error = TRY_LATER;
				return false;
			}

			return true;			
		}else{
			$this->error = _CATEGORY_NAME_EMPTY;
			return false;
		}
	}

	/***
	 *	Delete category 
	 *		@param $category_id - category ID
	 **/
	function CategoryDelete($category_id = "0", $category_order = "0") {

		// Block operation in demo mode
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;
		}

		$sql = "DELETE FROM ".DB_PREFIX."categories WHERE id = ".(int)$category_id."";
		if(database_void_query($sql)){
			$sql = "UPDATE ".DB_PREFIX."categories SET category_order = category_order - 1 WHERE category_order > ".(int)$category_order."";
			if(database_void_query($sql)){
				return true;    
			}                				   
		}

		return false;
	}

	/***
	 *	Move category (change priority order)
	 *		@param $category_id
	 *		@param $dir - direction
	 *		@param $category_order  - category order
	 **/
	function CategoryMove($category_id, $dir = "", $category_order = "") {		

		// Block operation in demo mode
		if(strtolower(SITE_MODE) == "demo"){ 
			$this->error = _OPERATION_BLOCKED;
			return false;
		}

		if(($dir == "") || ($category_order == "")) return false;		

		$sql = "SELECT * FROM ".DB_PREFIX."categories
				WHERE
					id <> '".(int)$category_id."' AND
					category_order ".(($dir == "up") ? "<" : ">")." ".(int)$category_order."
				ORDER BY category_order ".(($dir == "up") ? "DESC" : "ASC");
        if($category = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY)){
			$sql = "UPDATE ".DB_PREFIX."categories SET category_order = '".$category_order."' WHERE id = ".(int)$category['id']."";
			if(database_void_query($sql)){
				$sql = "UPDATE ".DB_PREFIX."categories SET category_order = '".$category['category_order']."' WHERE id = ".(int)$category_id."";				
				if(!database_void_query($sql)){
					$this->error = TRY_LATER;
					return false;					
				}
			}else{
				$this->error = TRY_LATER;
				return false;
			}
		}
		return true;		
	}

	/***
	 *	Draw all categories in dropdowm box
	 *		@param $category_id
	 **/
	function DrawCategorySelectBox($category_id = "", $post_placement = "left") {
		$disabled = ($post_placement == "top") ? "disabled='disabled'" : ""; 
		
		echo "<select name='category_id' id='category_id' ".$disabled.">";
		echo "<option value=''>-- select --</option>";
		$all_categories = self::GetAll(" category_order ASC", "categories", "");		                 
		for($i = 0; $i < $all_categories[1]; $i++){
			echo "<option value='".$all_categories[0][$i]['id']."'";
			echo ($all_categories[0][$i]['id'] == $category_id) ? " selected " : "";
			echo ">".$all_categories[0][$i]['category_name']."</option>";
		}
		echo "</select>";		
	}

	/***
	 *	Draw post placement in dropdowm box
	 *		@param $category_id
	 **/
	function DrawPostPlacementBox($post_placement = "") {
		echo "<select name='post_placement'>";
		echo "<option value='left' ".(($post_placement == "left") ? " selected " : "").">".LEFT."</option>";
		echo "<option value='top' ".(($post_placement == "top") ? " selected " : "").">".TOP."</option>";
		echo "</select>";		
	}

	/***
	 *	Draw all top links
	 *		@param $is_logged_in
	 *		@param $site_preview
	 **/
	function DrawTopCategory($is_logged_in = false, $site_preview = "0") {
		// If not logged in or site preview == 1 - draw category
		global $objSettings;
		
		echo "<li><a class='main_link' href='index.php'>"._HOME."</a></li>";
		if(!$is_logged_in || $site_preview){
		
			$categories = self::GetAllTop("");
			for($category_ind = 0; $category_ind < $categories[1]; $category_ind++) {
				// Draw current category link
				echo "&nbsp;".get_divider()."&nbsp;";
				
				if($objSettings->GetParameter("seo_urls") == "1"){
					// Use SEO optimized link
					echo "<li><a class='main_menu_link' href='posts/".$categories[0][$category_ind]['id']."/".$categories[0][$category_ind]['post_key'].".html".(($site_preview == "1") ? "?preview=1" : "")."'>".camel_case($categories[0][$category_ind]['post_link'])."</a></li>";					
				}else{
					echo "<li><a class='main_menu_link' href='index.php?page=posts&post_id=".$categories[0][$category_ind]['id'].(($site_preview == "1") ? "&preview=1" : "")."'>".camel_case($categories[0][$category_ind]['post_link'])."</a></li>";
				}
			}
		}
	}

	/***
	 *	Draw all categories as a tree (for left category)
	 *		@param $is_logged_in
	 *		@param $site_preview
	 **/
	function DrawTree($is_logged_in = false){
		global $objSettings;
		
		// Get all categories which have items (links to static posts)
		$categories = self::GetLeftCategories();		
	    $text_align = ($objSettings->GetDirection() == "ltr") ? "text-align: left;" : "text-align: right; padding-right:15px;";

		// Display all category titles (names) according to their order
		$cur_category = "";
		//$is_start = true;		
		
		echo "<table class='moduletable' width='100%'>";
		echo "<tr><th valign='top' style='".$text_align."'>".strtoupper(_CATEGORIES)."</th></tr>";		
		for($category_ind = 0; $category_ind < $categories[1]; $category_ind++) {				
			$category_links = self::GetLeftCategoryLinks($categories[0][$category_ind]['id']);
			// Start draw new category
			echo "<tr><td valign='top' style='".$text_align."'><a class='menu' href='index.php?page=posts&cat_id=".$categories[0][$category_ind]['id']."' class='main_category_link'>".camel_case($categories[0][$category_ind]['category_name'])." (".$category_links[1].")</a></td></tr>";			
        }
		echo "</table>";		

		// End drawing last category
		//if(!$is_start){
        //    echo "</table>";
        //    draw_block_bottom();
        //}
	}

}
?>