<?php

/**
 *	Class Languages
 *  -------------- 
 *  Description : encapsulates languages properties
 *  Updated	    : 17.05.2010
 *	Written by  : ApPHP
 *
 *	__construct
 *	__destruct
 *
 *	STATIC
 *	------------------
 *	GetDefaultLang
 *	LanguageExists
 *	
 *	PUBLIC
 *	------------------
 *	GetParameter
 *	DrawLanguage
 *	LanguageUpdate
 *	LanguageDelete
 *	GetAll
 *	LanguageMove
 *	
 *	PRIVATE
 *	-----------------
 *	CopyDataToNewLang
 *	DeleteDataOfLang
 *	
 *	
 *	
 **/

class Languages {

	private $id;
	protected $language;
	public $error;
	
	private $languages_count; 
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct() {

		$sql = "SELECT
					id, lang_name, abbreviation, lang_dir, icon_image
				FROM ".DB_PREFIX."languages";
		$this->language = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);
	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct(){
		// echo 'this object has been destroyed';
    }

	//==========================================================================
    // Static Methods
	//==========================================================================	
	/***
	 *	Get default language
	 **/
	static function GetDefaultLang(){
		$def_language = "en";
		$sql = "SELECT abbreviation FROM ".DB_PREFIX."languages";
		if($language = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY)){
			$def_language = $language['abbreviation'];					
		}
		return $def_language;
	}

	/***
	 *	Check if language exists
	 **/
	static function LanguageExists($lang_abbrev){
		$sql = "SELECT abbreviation FROM ".DB_PREFIX."languages WHERE abbreviation = '".$lang_abbrev."' ";
		if(database_query($sql, ROWS_ONLY) > 0){
			return true;
		}
		return false;
	}

	/***
	 *	Get language direction
	 **/
	static function GetLanguageDirection($lang_abbrev = ""){
		$lang_dir = "ltr";
		$sql = "SELECT lang_dir FROM ".DB_PREFIX."languages WHERE abbreviation = '".$lang_abbrev."'";
		if($language = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY)){
			$lang_dir = $language['lang_dir'];					
		}
		return $lang_dir;
	}

	//==========================================================================
    // Public Methods
	//==========================================================================
	/***
	 *	Get languages count
	 *	
	 **/
	public function GetLanguagesCount(){
		$sql = "SELECT COUNT(*) as cnt FROM ".DB_PREFIX."languages ";
		$language = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);        
		return $language['cnt'];
	}

	/***
	 *	Get language parameter
	 *	
	 **/
	public function GetParameter($param = ""){
		if(isset($this->language[$param])){
			return $this->language[$param];
		}else{
			return "";
		}
	}

	/***
	 *	Draw languages on the screen
	 *	
	 **/
	public function DrawLanguage(){
		global $defined_right, $defined_left;
		#echo "<pre>";
		#print_r($this->language);
		#echo "</pre>";
		$all_languages = Languages::GetAll();
		
		if($all_languages > 0){
		
			echo "<table width='99%' border='0' cellspacing='0' cellpadding='2' class='main_text'>
				<tr>
					<th align='".$defined_left."'>"._LANGUAGE_NAME."</th>
					<th width='12%'>"._ICON_IMAGE."</th>
					<th width='12%'>"._HDR_TEXT_DIRECTION."</th>
					<th width='10%'>"._ACTIONS."</th>
				</tr>
				<tr><td colspan='8' height='3px' nowrap='nowrap'>".draw_line("no_margin_line", IMAGE_DIRECTORY, false)."</td></tr>";
		
				for($i=0; $i < $all_languages[1]; $i++){
					echo "<tr ".highlight(0)." onmouseover='oldColor=this.style.backgroundColor;this.style.backgroundColor=\"#e1e1e1\";' onmouseout='this.style.backgroundColor=oldColor'>
							<td align='".$defined_left."'>".$all_languages[0][$i]['lang_name']." (".strtoupper($all_languages[0][$i]['abbreviation']).")</td>
							<td align='center'>".(file_exists("images/flags/".$all_languages[0][$i]['icon_image']) ? "<img src='images/flags/".$all_languages[0][$i]['icon_image']."' border='0' />" : "No Image")."</td>
							<td align='center'>".$all_languages[0][$i]['lang_dir']."</td>							
							<td align='center' nowrap>
								<a href='index.php?admin=language_edit' title='"._CLICK_TO_EDIT."'>"._EDIT_WORD."</a>
							</td>
						</tr>";
				}
			echo "</table>";
		}
	}


	/***
	 *	Edit language
	 *		@param $param - array of parameters
	 **/
	function LanguageUpdate($params = array()) {

		// Block operation in demo mode
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;
		}

		// Get input parameters
		if(!isset($params['id'])) 			$params['id'] = "0";
		if(!isset($params['lang_name'])) 	$params['lang_name'] = "";
		if(!isset($params['lang_dir']))     $params['lang_dir'] = "";
		if(!isset($params['lang_order'])) 	$params['lang_order'] = "";
		if(!isset($params['abbreviation'])) $params['abbreviation'] = "";
		if(!isset($params['icon_image'])) 	$params['icon_image'] = "";

		// Prevent creating of empty records in our 'languages' table
		if($params['lang_name'] == ""){
			$this->error = _LANG_NAME_EMPTY;
			return false;
		}else if($params['abbreviation'] == ""){
			$this->error = _LANG_ABBREV_EMPTY;
			return false;			
		}

		$sql = "UPDATE ".DB_PREFIX."languages SET
					lang_name = '".encode_text($params['lang_name'])."',
					lang_dir  = '".encode_text($params['lang_dir'])."',
					abbreviation = '".encode_text($params['abbreviation'])."',
					icon_image = '".encode_text($params['icon_image'])."'";
		$result = (database_void_query($sql) >= 0) ? true : false;
				
	
		if($result){
			return true;
		}else{
			$this->error = _TRY_LATER;
			return false;
		}				
	}

	/***
	 *	Get all active languages array
	 **/
	function GetAllActive() {		
		$sql = "SELECT
					id, lang_name, abbreviation, lang_dir, icon_image
				FROM ".DB_PREFIX."languages 
				ORDER BY lang_order ASC";			
		
		return database_query($sql, DATA_AND_ROWS);
	}

	/***
	 *	Get all languages array
	 *		@param $order - order clause
	 *		@param $join_table - join tables
	 **/
	function GetAll($order = "", $join_table = "", $where = "") {
		
		$where_clause = ($where != "") ? " AND ".$where : "";
		
		// Build ORDER BY CLAUSE
		if($order=="")$order_clause = "";
		else $order_clause = "ORDER BY $order";		

		// Build JOIN clause
		$join_clause = "";
		$join_select_fields = "";
		
		$sql = "SELECT
					id, lang_name, abbreviation, lang_dir, icon_image
				FROM ".DB_PREFIX."languages 
					$join_clause
					WHERE 1=1 $where_clause
				$order_clause";			
		
		return database_query($sql, DATA_AND_ROWS);
	}

	/***
	 *	Move language (change priority order)
	 *		@param $lang_id
	 *		@param $dir - direction
	 *		@param $lang_order  - menu order
	 **/
	function LanguageMove($lang_id, $dir = "", $lang_order = "") {		

		// Block operation in demo mode
		if(strtolower(SITE_MODE) == "demo"){ 
			$this->error = _OPERATION_BLOCKED;
			return false;
		}

		if(($dir == "") || ($lang_order == "")) return false;		

		$sql = "SELECT * FROM ".DB_PREFIX."languages
				WHERE id <> '".(int)$lang_id."' AND lang_order ".(($dir == "up") ? "<" : ">")." ".(int)$lang_order."
				ORDER BY lang_order ".(($dir == "up") ? "DESC" : "ASC");
        if($language = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY)){
			$sql = "UPDATE ".DB_PREFIX."languages SET lang_order = '".$lang_order."' WHERE id = ".(int)$language['id']."";
			if(database_void_query($sql)){
				$sql = "UPDATE ".DB_PREFIX."languages SET lang_order = '".$language['lang_order']."' WHERE id = ".(int)$lang_id."";				
				if(!database_void_query($sql)){
					$this->error = _TRY_LATER;
					return false;					
				}
			}else{
				$this->error = _TRY_LATER;
				return false;
			}
		}
		return true;		
	}

	//==========================================================================
    // Private Methods
	//==========================================================================
	
	/***
	 *	Copy default data for new language
	 *		@param $string
	 **/
	private function CopyDataToNewLang($from_abbrev, $to_abbrev){
		
		// clone data for Menus table
		$sql = "INSERT INTO ".DB_PREFIX."menus (language_id, menu_name, menu_order)
				(SELECT '".$to_abbrev."', menu_name, menu_order FROM ".DB_PREFIX."menus WHERE language_id = '".$from_abbrev."')";
		database_void_query($sql);					

		// clone data for Vocabulary table
		$sql = "INSERT INTO ".DB_PREFIX."vocabulary (language_id, key_value, key_text)
				(SELECT '".$to_abbrev."', key_value, key_text FROM ".DB_PREFIX."vocabulary WHERE language_id = '".$from_abbrev."')";
		database_void_query($sql);					

		// clone data for Static Pages table
		$sql = "INSERT INTO ".DB_PREFIX."pages (language_id, page_key, page_title, page_text, menu_id, menu_link, menu_placement)
				(SELECT '".$to_abbrev."', page_key, page_title, page_text, menu_id, menu_link, menu_placement FROM ".DB_PREFIX."pages WHERE language_id = '".$from_abbrev."')";
		database_void_query($sql);					

		// copy default messages.inc.php for new language
		$source_file = "include/messages.".$from_abbrev.".inc.php";
		$desfination_file = "include/messages.".$to_abbrev.".inc.php";
		@copy($source_file, $desfination_file);
	}
	
	/***
	 *	Delete data of language
	 *		@param $string
	 **/
	private function DeleteDataOfLang($lang_abbrev){
	
		// delete data from Menus table	
		$sql = "DELETE FROM ".DB_PREFIX."menus WHERE language_id = '".$lang_abbrev."'";
		database_void_query($sql);					

		// delete data from Vocabulary table	
		$sql = "DELETE FROM ".DB_PREFIX."vocabulary WHERE language_id = '".$lang_abbrev."'";
		database_void_query($sql);					

		// delete data from Pages table	
		$sql = "DELETE FROM ".DB_PREFIX."pages WHERE language_id = '".$lang_abbrev."'";
		database_void_query($sql);					

		// delete language file
		@unlink("include/messages.".$lang_abbrev.".inc.php");
	}
	
	
}
?>