<?php

/**
 *	Class Login
 *  -------------- 
 *  Description : encapsulates login properties
 *  Updated     : 17.01.2010	
 *	Written by  : ApPHP
 *	
 **/

class Login {

	private $wrong_login;
	private $ip_address_blocked;
	private $messages;
	private $active_menu_count;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct()
	{
		
		$submit_login  = isset($_POST['submit_login']) ? prepare_input($_POST['submit_login']) : "";
		$submit_logout = isset($_POST['submit_logout']) ? prepare_input($_POST['submit_logout']) : "";
		$user_name 	   = (isset($_POST['user_name']) && check_input($_POST['user_name'])) ? prepare_input($_POST['user_name'], "high", "", "", true) : "";
		$password 	   = (isset($_POST['password']) && check_input($_POST['user_name'])) ? prepare_input($_POST['password'], "high", "", "", true) : "";
		
		$this->wrong_login = false;		
		if(!$this->IsLoggedIn()){
			if($submit_login == "login"){
				if(empty($user_name) || empty($password)){
					$this->wrong_login = true;							
				}else{
					$this->DoLogin($user_name, $password);
				}
			}			
		}else if ($submit_logout == "logout"){
			$this->DoLogout();
		}
		$this->active_menu_count = 0;
	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct()
	{
		// echo 'this object has been destroyed';
    }

	/***
	 * 	Do login
	 * 		@param $user_name - system name of user
	 * 		@param $password - password of user
	 * 		@param $do_redirect - prepare redirect or not
	 **/
	private function DoLogin($user_name, $password, $do_redirect = true)
	{
		$ip_address = "";
		if(isset($_SERVER['HTTP_X_FORWARD_FOR'])){
			$ip_address = $_SERVER['HTTP_X_FORWARD_FOR'];
		}else if(isset($_SERVER['REMOTE_ADDR'])){
			$ip_address = $_SERVER['REMOTE_ADDR'];
		}
		
		if($account_information = $this->GetAccountInformation($user_name, $password)){
			$this->SetSessionVariables($account_information);

			if($this->IsLoggedInAsAuthor() && $this->IpAddressBlocked($ip_address)){
				$this->DoLogout();
				$this->ip_address_blocked = true;
				$do_redirect = false;
			}	

			$this->UpdateAccountInfo($account_information);
			if($do_redirect) {				
				header("Location: index.php");
				exit;
			}
		}else{
			$this->wrong_login = true;
		}
	}
	
	/***
	 * 	Return if login was wrong
	 * 	
	 **/
	function IsWrongLogin()
	{
		return ($this->wrong_login == true) ? true : false;	
	}

	/***
	 * 	Return if IP address was blocked
	 * 	
	 **/
	function IsIpAddressBlocked()
	{
		return ($this->ip_address_blocked == true) ? true : false;	
	}
	
	/***
	 * 	Destroys the session and returns to the default page
	 * 	
	 **/
	function DoLogout()
	{
		$GLOBALS['user_session']->EndSession();
	}

	/***
	 * 	Check Ip Address
	 * 		@param $ip_address
	 **/
	private function IpAddressBlocked($ip_address)
	{
		$sql = "SELECT ban_ip 
				FROM ".DB_PREFIX."banlist 
				WHERE ban_ip = '" . $ip_address . "'";			
		return database_query($sql, ROWS_ONLY);		
	}
	
	
	/***
	 * 	Gets the account information
	 * 		@param $user_name - system name of user
	 * 		@param $password - password of user
	 **/
	private function GetAccountInformation($user_name, $password)
	{
		if(PASSWORDS_ENCRYPTION){			
			if(PASSWORDS_ENCRYPTION_TYPE == "AES"){
				$password = "AES_ENCRYPT('".$password."', '".PASSWORDS_ENCRYPT_KEY."')";
			}else if(PASSWORDS_ENCRYPTION_TYPE == "MD5"){
				$password = "MD5('".$password."')";
			}	
		}else{
			$password = "'".$password."'";
		}
		$sql = "SELECT ".DB_PREFIX."accounts.*, user_name AS account_name
				FROM ".DB_PREFIX."accounts
				WHERE 
					 user_name = '" . $user_name . "' AND 
					 password = ".$password;			
		$sql .= " AND is_active = 1";
		
		return database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);
	}

	/***
	 * 	Get last login time
	 * 		@return last login time
	 **/
	function GetLastLoginTime()
	{
		$last_login = $GLOBALS['user_session']->GetSessionVariable("session_last_login");

		if($last_login != "" && $last_login != "0000-00-00 00:00:00" && $last_login != "NULL"){
			return $last_login;
		}else{
			return "--";
		}
	}

	/***
	 * 	Checks to see if the user is logged in
	 * 		@return a 1 if the user is logged in, 0 otherwise
	 **/
	public function IsLoggedIn()
	{
		$logged = $GLOBALS['user_session']->GetSessionVariable('session_account_logged');
		$id = $GLOBALS['user_session']->GetSessionVariable('session_account_id');
		return ($logged == str_replace("modules/wysiwyg/addons/imagelibrary/", "", $this->getUniqueUrl()).$id);
	}

	/***
	 * 	Checks to see if the user is logged in as a specific account type
	 * 		@return true if the user is logged in as specified account type,
	 * 			    false otherwise
	 **/
	function IsLoggedInAs()
	{
		if (!$this->IsLoggedIn()) return false;

		$account_type = $GLOBALS['user_session']->GetSessionVariable("session_account_type");

		$types = func_get_args();
		foreach ($types as $type) {
			if ($account_type == $type) return true;
		}

		return false;
	}

	/***
	 * 	Checks to see if the admin is logged in as a specific account type
	 * 		@return true if the admin is logged in as specified account type,
	 * 			    false otherwise
	 **/
	function IsLoggedInAsAdmin()
	{
		if (!$this->IsLoggedIn()) return false;
		$account_type = $GLOBALS['user_session']->GetSessionVariable("session_account_type");
		if ($account_type == "admin") return true;
		return false;
	}

	/***
	 * 	Checks to see if the admin is logged in as a specific account type
	 * 		@return true if the admin is logged in as specified account type,
	 * 			    false otherwise
	 **/
	function IsLoggedInAsAuthor()
	{
		if(!$this->IsLoggedIn()) return false;
		$account_type = $GLOBALS['user_session']->GetSessionVariable("session_account_type");
		if ($account_type == "author") return true;
		return false;
	}
	
	/***
	 * 	Returns logged user type
	 **/
	function GetLoggedType()
	{
		return $GLOBALS['user_session']->GetSessionVariable("session_account_type");
	}

	/***
	 * 	Returns logged user name
	 **/
	function GetLoggedName()
	{
		return $GLOBALS['user_session']->GetSessionVariable("session_user_name");
	}
	
	/***
	 * 	Returns logged user ID
	 **/
	function GetLoggedID()
	{
		return $GLOBALS['user_session']->GetSessionVariable("session_account_id");
	}	

	/***
	 * 	Sets the session variables and performs the login
	 * 		@param $account_information - array
	 **/
	function SetSessionVariables($account_information)
	{		
		$GLOBALS['user_session']->SetSessionVariable("session_account_logged", (($account_information['id']) ? $this->GetUniqueUrl().$account_information['id'] : false));			
		$GLOBALS['user_session']->SetSessionVariable("session_account_id", $account_information['id']);
		$GLOBALS['user_session']->SetSessionVariable("session_user_name", $account_information['user_name']);
		$GLOBALS['user_session']->SetSessionVariable("session_account_type", (isset($account_information['account_type']) ? $account_information['account_type'] : "author"));
		$GLOBALS['user_session']->SetSessionVariable("session_last_login", date("Y-m-d H:i:s"));
	}
	
	/**
	 *  Get unique URL 
	 **/
	function GetUniqueUrl()
	{
		$port = "";
		$http_host = $_SERVER['HTTP_HOST'];
		if(isset($_SERVER['SERVER_PORT']) && $_SERVER['SERVER_PORT'] != "80"){
			if(!strpos($_SERVER['HTTP_HOST'], ":")){
				$port = ":".$_SERVER['SERVER_PORT'];
			}
		}	
		$folder = substr($_SERVER['SCRIPT_NAME'], 0, strrpos($_SERVER['SCRIPT_NAME'], "/")+1);	
		return $http_host.$port.$folder;
	}	
	
	/***
	 * 	Returns count of active menus
	 * 		@return number on menus
	 **/
	function GetActiveMenuCount()
	{
		return $this->active_menu_count;
	}
	
	
	/***
	 * 	Update Account Info	
	 * 		@param $account_information - array
	 **/
	function UpdateAccountInfo($account_information)
	{
		$sql = "UPDATE ".DB_PREFIX."accounts
				SET last_login = '".date("Y-m-d H:i:s")."'
				WHERE id = ".(int)$account_information['id']."";
		return database_void_query($sql);
	}	

	/***
	 * 	Draws the login welcome message
	 * 	
	 **/	
	function DrawWelcomeMessage()
	{
		if($GLOBALS['user_session']->GetSessionVariable("session_user_name") != "")
			echo "<b>".ucfirst($GLOBALS['user_session']->GetSessionVariable("session_user_name"))." "._WELCOME_ADMIN."</b>";
	}

	/***
	 * 	Draws the login links and logout form
	 * 	
	 **/
	function DrawLoginLinks()
	{
		global $objSettings;
		
		$menu_index = "0";
		$text_align = ($objSettings->GetDirection() == "ltr") ? "text-align: left;" : "text-align: right; padding-right:15px;";
		
		// ---------------------------------------------------------------------
		// MAIN ADMIN LINKS
		if($this->IsLoggedIn()) {
			draw_block_top(_MENUS.": [ <a id='lnk_all_open' href='javascript:void(0);' onclick='javascript:toggle_menus(1)'>"._OPEN."</a> | <a id='lnk_all_close' href='javascript:void(0);' onclick='javascript:toggle_menus(0)'>"._CLOSE."</a> ]"); draw_block_bottom();

			draw_block_top(_GENERAL, $menu_index++, "maximized");        
				echo "<ul>";
				echo "<li><a href='index.php?admin=home'>"._HOME."</a></li>";
				echo "<li><a href='index.php?preview=yes'>"._PREVIEW."</a></li>";
				if ($this->IsLoggedInAsAdmin()) echo "<li><a href='index.php?admin=settings'>"._SETTINGS."</a></li>";
				if ($this->IsLoggedInAsAdmin()) echo "<li><a href='index.php?admin=ban_list'>"._BAN_LIST."</a></li>";
				echo "</ul>";
			draw_block_bottom();

			draw_block_top(_ACCOUNTS_MANAGEMENT, $menu_index++);
				echo "<ul>";
				echo "<li><a href='index.php?admin=my_account'>"._MY_ACCOUNT."</a></li>";
				if($this->IsLoggedInAsAdmin()) echo "<li><a class='main_menu_link' href='index.php?admin=authors_management'>"._AUTHORS."</a></li>";
				echo "</ul>";
			draw_block_bottom();

			draw_block_top(_POSTS, $menu_index++);
				echo "<div style='font-size:12px;'>";
				if ($this->IsLoggedInAsAdmin()){
					echo _CATEGORIES_MANAGEMENT;
					echo "<ul>";
					echo "<li><a href='index.php?admin=categories_add'>"._CATEGORIES_ADD_NEW."</a></li>";
					echo "<li><a href='index.php?admin=categories'>"._CATEGORIES_EDIT."</a></li>";
					echo "</ul>";								
				}
	
			    echo _POSTS_MANAGEMENT;
				echo "<ul>";			
				echo "<li><a class='main_menu_link' href='index.php?admin=posts_add'>"._POST_ADD_NEW."</a></li>";
				echo "<li><a class='main_menu_link' href='index.php?admin=posts'>"._POSTS_EDIT."</a></li>";				
				echo "</ul>";
				echo "</div>";
			draw_block_bottom();

			if($this->IsLoggedInAsAdmin()){
				draw_block_top(_LANGUAGE_SETTINGS, $menu_index++);
					echo "<ul>";			
					echo "<li><a class='main_menu_link' href='index.php?admin=language'>"._LANGUAGE."</a></li>";
					echo "<li><a class='main_menu_link' href='index.php?admin=vocabulary&filter_by=A'>"._VOCABULARY."</a></li>";
					echo "</ul>";
				draw_block_bottom();				
			}

			if($this->IsLoggedInAsAdmin()){			
				draw_block_top(_MODULES, $menu_index++);
					echo "<ul>";			
					echo "<li><a style='margin-bottom:5px' href='index.php?admin=modules'>"._MODULES_MANAGEMENT."</a></li>";				
					echo "</ul>";
					
					// MODULES
					$sql = "SELECT * FROM ".DB_PREFIX."modules WHERE is_installed = 1";
					$modules = database_query($sql, DATA_AND_ROWS, ALL_ROWS);
					
					echo "<div style='font-size:12px;'>";
					for($i=0; $i < $modules[1]; $i++){				
						if($modules[0][$i]['name'] == "gallery"){
							echo _GALLERY;
							echo "<ul>";
							echo "<li><a class='main_menu_link' href='index.php?admin=mod_gallery_settings'>"._GALLERY_SETTINGS."</a></li>";
							echo "<li><a class='main_menu_link' href='index.php?admin=mod_gallery_management'>"._GALLERY_MANAGEMENT."</a></li>";
							echo "</ul>";
						}
						if($modules[0][$i]['name'] == "resources"){
							echo _RESOURCES;
							echo "<ul>";
							echo "<li><a class='main_menu_link' href='index.php?admin=mod_resources_settings'>"._RESOURCES_SETTINGS."</a></li>";
							echo "<li><a class='main_menu_link' href='index.php?admin=mod_resources_management'>"._RESOURCES_MANAGEMENT."</a></li>";
							echo "</ul>";
						}				
						if($modules[0][$i]['name'] == "adsense"){
							echo _ADSENSE;
							echo "<ul>";
							echo "<li><a class='main_menu_link' href='index.php?admin=mod_adsense_settings'>"._ADSENSE_SETTINGS."</a></li>";
							echo "</ul>";
						}										
						if($modules[0][$i]['name'] == "contact"){
							echo _CONTACT;
							echo "<ul>";
							echo "<li><a class='main_menu_link' href='index.php?admin=mod_contact_settings'>"._CONTACT_SETTINGS."</a></li>";
							echo "</ul>";
						}				
						if($modules[0][$i]['name'] == "comments"){
							echo _COMMENTS;
							echo "<ul>";
							echo "<li><a class='main_menu_link' href='index.php?admin=mod_comments_settings'>"._COMMENTS_SETTINGS."</a></li>";
							echo "</ul>";
						}				
						if($modules[0][$i]['name'] == "backup"){
							echo _BACKUP_AND_RESTORE;
							echo "<ul>";
							echo "<li><a class='main_menu_link' href='index.php?admin=mod_backup_installation'>"._BACKUP_INSTALLATION."</a></li>";
							echo "<li><a class='main_menu_link' href='index.php?admin=mod_backup_restore'>"._BACKUP_RESTORE."</a></li>";
							echo "</ul>";
						}				
					}
					echo "</div>";
				draw_block_bottom();
			}
		}
	
		// Logout
		if ($this->IsLoggedIn()){
			draw_block_top_empty();
			echo "<form action='index.php' method='post'>&nbsp;&nbsp;<input type='hidden' name='submit_logout' value='logout' /><input class='form_button' type='submit' name='btnLogout' value='"._BUTTON_LOGOUT."' />&nbsp;&nbsp;</form>";
			draw_block_bottom();						
            
		}
		
		echo "<br>";
		
		$this->active_menu_count = $menu_index;
	}
}
?>