<?php

/**
 *	Class MicroGrid
 *  -------------- 
 *  Description : encapsulates grid operations & properties
 *  Updated	    : 18.07.2010
 *	Written by  : ApPHP
 *	Version     : 1.3.5
 *
 *  1.3.5
 *      - fixed issue with apostrophes in Details mode for labels
 *      - reduce GetDataDecoded GetDataEncoded
 *  1.3.4
 *      - GetDataDecodedText for textzrea
 *      - alignment abd blanks for filtering
 *      - add new validation type validation_maxlength + _FIELD_LENGTH_EXCEEDED : _FIELD_ has exceeded the maximum allowed size: _SIZE_ characters! Please re-enter.
 *      - using of uniquie prexif for some datagrids in one page:
 *      	- $this->uPrefix = "htl_"; +
 *      	- $action 	= MicroGrid::GetParameter('action', true, 'htl_');
 *  1.3.3
 *      - wrong <br /> in drawEdit if there are blocks
 *      - for image if($random_name == "true" || $random_name === true){
 *      - added for Post.. function $this->lastInsertId
 *      - added "formate"=>"strip_tags"
 *  1.3.2
 *  	- fixed appostrofes problem for textboxes
 *  	- $visible = isset($val['visible']) ? $val['visible'] : true;
 *  	- bug fixed with arrEditModeFields in UpdateRecord()
 *  	- for filter fields: echo $key.": &nbsp;
 *  1.3.1 
 *  	- _FIELD_MUST_BE_IP_ADDRESS : *	'_FIELD_ must be a valid IP Addrss! Please re-enter.'
 *      - legend isset()
 *      - wrong displaing hidden field for edit mode if fieldset used
 *	
 **/

class MicroGrid {
		
	protected $debug = false;
		
	protected $tableName;
	protected $primaryKey;
	protected $dataSet;
	protected $lastInsertId;
	
	protected $formActionURL;
	protected $params;
	protected $actions;
	protected $actionIcons;
	protected $errorField;
	protected $arrErrors;
	protected $arrWarnings;
	protected $arrSQLs;

	protected $languageId;
	protected $allowLanguages;
	
	protected $VIEW_MODE_SQL;
	protected $EDIT_MODE_SQL;	
	protected $DETAILS_MODE_SQL;
	protected $WHERE_CLAUSE;
	protected $ORDER_CLAUSE;
	protected $LIMIT_CLAUSE;
	
	protected $arrViewModeFields;	
	protected $arrAddModeFields;	
	protected $arrEditModeFields;
	protected $arrDetailsModeFields;
	
	protected $isHtmlEncoding;
	
	protected $isAlterColorsAllowed;
	
	protected $isPagingAllowed;
	protected $pageSize;
	
	protected $isSortingAllowed;
	
	protected $isFilteringAllowed;
	protected $arrFilteringFields;
	
	protected $arrImagesFields;
	
	private $startTime;
	private $finishTime;
	
	//TODO
	// - add try catch + error
	// returns errors
	
	protected $uPrefix;
	
	public $error;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct($id = "") {

		$this->params = array();

		$this->isHtmlEncoding = false;
		
		$this->uPrefix          = "";
		
		$this->primaryKey       = "";
		$this->tableName        = "";
		$this->formActionURL    = "";
		
		$this->languageId  	    = "";
		$this->allowLanguages   = false;

		$this->VIEW_MODE_SQL    = "";
		$this->EDIT_MODE_SQL    = "";
		$this->DETAILS_MODE_SQL = "";
		$this->WHERE_CLAUSE     = "";
		$this->ORDER_CLAUSE     = "";	
		$this->LIMIT_CLAUSE     = "";
		
		$this->arrViewModeFields    = array();
		$this->arrAddModeFields     = array();
		$this->arrEditModeFields    = array();
		$this->arrDetailsModeFields = array();
		$this->arrFilterModeFields  = array();
		
		$this->actions = array("add"=>true, "edit"=>true, "details"=>true, "delete"=>true);
		$this->actionIcons = false;
		
		$this->dataSet = array();
		$this->lastInsertId = "";
		$this->curRecordId = "";
		$this->arrErrors = array();
		$this->arrWarnings = array();
		$this->arrSQLs = array();
		$this->error = "";
		$this->errorField = "";
		
		$this->isAlterColorsAllowed = true;
		
		$this->isPagingAllowed = true;
		$this->pageSize = 10;
		
		$this->isSortingAllowed = true;

		$this->isFilteringAllowed = true;
		$this->arrFilteringFields = array();
		
		$this->arrImagesFields = array();
		
	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct()
	{
		// echo 'this object has been destroyed';
    }


	/***
	 *	Get all DataSet array
	 **/
	function GetAll($order_clause = "", $limit_clause = "") {
		$sql = $this->VIEW_MODE_SQL." ".$this->WHERE_CLAUSE." ".$order_clause." ".$limit_clause;
		if($this->debug) $this->arrSQLs['select_get_all'] = $sql;					
		return database_query($sql, DATA_AND_ROWS);
	}

	/***
	 *	Get info by ID
	 **/
	function GetInfoByID($key = "") {
		$sql = "SELECT * FROM ".$this->tableName." WHERE ".$this->primaryKey."=".(int)$key." LIMIT 0, 1";
		return database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);
	}
	
	/***
	 *	Get one record
	 **/
	function GetRecord($sql = "") {
		return database_query($sql, DATA_AND_ROWS, FIRST_ROW_ONLY);
	}
	
	/***************************************************************************
	 *
	 *	ADD NEW RECORD
	 *	
	 **************************************************************************/
	function AddRecord() {
		
		//----------------------------------------------------------------------
		// block if this is a demo mode
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;				
		}
		
		//----------------------------------------------------------------------
		// fields data validation
		if(!$this->FieldsValidation($this->arrAddModeFields)){
			return false;
		}		

		//----------------------------------------------------------------------
		// pre addition check
		if(!$this->PreAddRecord()){
			return false;
		}

		//----------------------------------------------------------------------
		// prepare (handle) uploaded files
		$arrUploadedFiles = $this->UploadFileImage("add");
		
		//----------------------------------------------------------------------
		// prepare INSERT SQL
		$sql = "INSERT INTO `".$this->tableName."`(";
			$sql .= $this->primaryKey;
			foreach($this->params as $key => $val){
				if(array_key_exists($key, $this->arrAddModeFields)){
					$sql .= ", `".$key."`";
				}else{
					foreach($this->arrAddModeFields as $v_key => $v_val){
						if(array_key_exists($key, $v_val)){
							$sql .= ", `".$key."`";
						}							
					}
				}
			}
			foreach($arrUploadedFiles as $key => $val){
				$sql .= ", `".$key."`";
			}
		$sql .= ") VALUES (";
			$sql .= "NULL";
			foreach($this->params as $key => $val){
				if(array_key_exists($key, $this->arrAddModeFields)){
					if($this->arrAddModeFields[$key]['type'] == "password"){
						$password_value = $this->CryptPasswordValue($key, $this->arrAddModeFields[$key], $val);
						$sql .= ", ".$password_value;
					}else{
						$sql .= ", '".encode_text($val)."'";
					}										
				}else{
					foreach($this->arrAddModeFields as $v_key => $v_val){
						if(array_key_exists($key, $v_val)){
							if($v_val[$key]['type'] == "password"){
								$password_value = $this->CryptPasswordValue($key, $v_val[$key], $val);
								$sql .= ", ".$password_value;
							}else{
								$sql .= ", '".encode_text($val)."'";
							}					
						}							
					}
				}
			}
			foreach($arrUploadedFiles as $key => $val){
				$sql .= ", '".$val."'";
			}
		$sql .= ")";
		
		if(!database_void_query($sql)){
			if($this->debug) $this->arrErrors['insert_sql'] = $sql."<br />".database_error();
			if($this->debug) $this->arrSQLs['insert_sql'] = $sql;
			$this->error = _TRY_LATER;
			return false;
		}else{
			$this->lastInsertId = database_insert_id();
			$this->PostAddRecord();
			return true;
		}		
	}

	/***************************************************************************
	 *
	 *	UPDATE RECORD
	 *	
	 **************************************************************************/
	function UpdateRecord($rid = "0") {
		
		//----------------------------------------------------------------------
		// block if this is a demo mode
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;				
		}		
		$this->curRecordId = $rid;
			
		//----------------------------------------------------------------------
		// check if we work with valid record
		if($rid == "0" || $rid == "" || !is_numeric($rid)){
			$this->error = _WRONG_PARAMETER_PASSED;
			return false;
		}		

		//----------------------------------------------------------------------
		// fields data validation
		if(!$this->FieldsValidation($this->arrEditModeFields)){
			return false;
		}		

		//----------------------------------------------------------------------
		// pre updating check
		if(!$this->PreUpdateRecord()){
			return false;
		}

		//----------------------------------------------------------------------
		// prepare (handle) uploaded files
		$arrUploadedFiles = $this->UploadFileImage("edit");
		
		//----------------------------------------------------------------------
		// update
		$sql = "UPDATE `".$this->tableName."` SET ";
			$fields_count = 0;
			foreach($this->params as $key => $val){
				if(array_key_exists($key, $this->arrEditModeFields)){
					if($fields_count++ > 0) $sql .= ",";
					if($this->arrEditModeFields[$key]['type'] == "password"){
						$password_value = $this->CryptPasswordValue($key, $this->arrEditModeFields[$key], $val);
						$sql .= "`".$key."` = ".$password_value;
					}else{
						$sql .= "`".$key."` = '".encode_text($val)."'";					
					}
				}else{
					foreach($this->arrEditModeFields as $v_key => $v_val){
						if(array_key_exists($key, $v_val)){
							if($fields_count++ > 0) $sql .= ",";
							if($v_val[$key]['type'] == "password"){
								$password_value = $this->CryptPasswordValue($key, $v_val[$key], $val);
								$sql .= "`".$key."` = ".$password_value;					
							}else{
								$sql .= "`".$key."` = '".encode_text($val)."'";					
							}					
						}							
					}
				}
			}				
			foreach($arrUploadedFiles as $key => $val){
				if($fields_count > 0) $sql .= ",";
				$sql .= "`".$key."` = '".encode_text($val)."'";					
			}
		$sql .= " WHERE `".$this->primaryKey."`=".(int)$rid;
		
		if($this->debug) $this->arrSQLs['update_sql'] = $sql;
		if(database_void_query($sql) < 0){
			if($this->debug) $this->arrErrors['update_sql'] = database_error();						
			$this->error = _TRY_LATER;
			return false;
		}else{
			$this->PostUpdateRecord();
			return true;
		}	
		
	}

	/***
	 *	Delete record
	 **/
	function DeleteRecord($rid = "")
	{
		//----------------------------------------------------------------------
		// check if rid is not empty
		if($rid == ""){				
			$this->error = _WRONG_PARAMETER_PASSED;
			return false;
		}
		$this->curRecordId = $rid;
		
		//----------------------------------------------------------------------
		// block if this is a demo mode
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;				
		}
		
		//----------------------------------------------------------------------
		// pre deleting check
		if(!$this->PreDeleteRecord()){
			return false;
		}

		$this->PrepareImagesArray($rid);

		//----------------------------------------------------------------------
		// delete
		$sql = "DELETE FROM ".$this->tableName." WHERE ".$this->primaryKey." = ".(int)$rid."";
		if(database_void_query($sql) > 0){
			$this->DeleteImages();
			if($this->debug) $this->arrSQLs['delete_sql'] = $sql;
			$this->PostDeleteRecord();
			return true;
		}else{
			if($this->debug) $this->arrErrors['delete_sql'] = $sql."<br />".database_error();
			$this->error = _TRY_LATER;
			return false;
		}
	}
	
	/***
	 *	Pre methods
	 **/
	function PreAddRecord(){
		return true;
	}

	function PreUpdateRecord(){
		return true;
	}

	function PreDeleteRecord(){
		return true;
	}

	/***
	 *	Post methods
	 **/
	function PostAddRecord(){
		// $this->lastInsertId - currently added record
	}

	function PostUpdateRecord(){
		// $this->curRecordId - currently updated record
	}

	function PostDeleteRecord(){
		// $this->curRecordId - currently deleted record
	}

	/***********************************************************************
	 *
	 *	Draw View Mode
	 *	
	 ***********************************************************************/
	function DrawViewMode()
	{
		global $defined_left, $defined_right;
		
		$this->IncludeJSFunctions();		
		
		$sorting_fields  = self::GetParameter('sorting_fields');
		$sorting_types   = self::GetParameter('sorting_types');
		$page 			 = self::GetParameter('page');
		$total_pages	 = $page;

		$operation 		 = self::GetParameter('operation');
		$operation_type  = self::GetParameter('operation_type');
		$operation_field = self::GetParameter('operation_field');
		
		$search_status   = self::GetParameter('search_status');
		
		$concat_sign 	 = (preg_match('/\?/', $this->formActionURL) ? "&" : "?");		
		$colspan 		 = count($this->arrViewModeFields)+1;
		$start_row 		 = 0;
		$total_records 	 = 0;

		// prepare changing of language
		if($operation == "change_language" && $operation_type != ""){
			$this->languageId = $operation_type;
		}
			
		// prepare sorting data
		if($this->isSortingAllowed){
			if($operation == "sorting"){
				if($sorting_fields != "" && $sorting_types != ""){
					if(strtolower($sorting_types) == "asc") $sorting_types = "DESC";
					else $sorting_types = "ASC";
					$this->ORDER_CLAUSE = " ORDER BY ".$sorting_fields." ".$sorting_types." ";
				}else{
					$sorting_types = "ASC";
				}
			}else{
				if($sorting_fields != "" && $sorting_types != ""){
					$this->ORDER_CLAUSE = " ORDER BY ".$sorting_fields." ".$sorting_types." ";	
				}				
			}
		}
		
		// prepare filtering data
		if($this->isFilteringAllowed){
			if($search_status == "active"){
				if($this->WHERE_CLAUSE == "") $this->WHERE_CLAUSE .= " WHERE 1=1 ";
				$count = 0;
				foreach($this->arrFilteringFields as $key => $val){					
					if(self::GetParameter("filter_by_".$val['table'].$val['field'], false) !== ""){
						$sign = "="; $sign_start = ""; $sign_end = "";
						if($val['sign'] == "="){
							$sign = "=";
						}else if($val['sign'] == "like%"){
							$sign = "LIKE";
							$sign_end = "%";
						}else if($val['sign'] == "%like"){
							$sign = "LIKE";
							$sign_start = "%";
						}else if($val['sign'] == "%like%"){
							$sign = "LIKE";
							$sign_start = "%";
							$sign_end = "%";
						}
						$key_value = self::GetParameter("filter_by_".$val['table'].$val['field'], false);
						if(isset($val['table']) && $val['table'] != "") $field_name = $val['table'].".".$val['field'];
						else $field_name = $val['field'];
						$this->WHERE_CLAUSE .= " AND ".$field_name." ".$sign." '".$sign_start.$key_value.$sign_end."' ";						
					}
				}
			}			
		}		

		// prepare paging data
		if($this->isPagingAllowed){
			if(!is_numeric($page) || (int)$page <= 0) $page = 1;
			$sql = $this->VIEW_MODE_SQL." ".$this->WHERE_CLAUSE;
			$total_records = (int)database_query($sql, ROWS_ONLY);
			if($this->pageSize == 0) $this->pageSize = "10";
			$total_pages = (int)($total_records / $this->pageSize);
			// when you back from other languages where more pages than on current
			if($page > ($total_pages+1)) $page = 1; 
			if(($total_records % $this->pageSize) != 0) $total_pages++;
			$start_row = ($page - 1) * $this->pageSize;				
		}
		
		$arrRecords = $this->GetAll($this->ORDER_CLAUSE, "LIMIT ".$start_row.", ".(int)$this->pageSize);
		if($this->allowLanguages) $arrLanguages = Languages::GetAllActive();
		if(!$this->isPagingAllowed){
			$total_records = $arrRecords[1];
		}
		
		echo "<form name='frmMicroGrid_".$this->tableName."' id='frmMicroGrid_".$this->tableName."' action='".$this->formActionURL."' method='post'>\n";
		echo draw_hidden_field($this->uPrefix."mg_action", "view")."\n";
		echo draw_hidden_field("mg_rid", "")."\n";
		echo draw_hidden_field("mg_sorting_fields", $sorting_fields)."\n";
		echo draw_hidden_field("mg_sorting_types", $sorting_types)."\n";
		echo draw_hidden_field("mg_page", $page)."\n";
		echo draw_hidden_field("mg_operation", $operation)."\n";
		echo draw_hidden_field("mg_operation_type", $operation_type)."\n";
		echo draw_hidden_field("mg_operation_field", $operation_field)."\n";
		echo draw_hidden_field("mg_search_status", $search_status)."\n";
		echo draw_hidden_field("mg_language_id", $this->languageId)."\n";

		if($this->actions['add'] || $this->allowLanguages){
			echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>
				<tr>
					<td align='".$defined_left."'>";
					if($this->actions['add']) echo "<input class='form_button' type='button' name='btnAddNew' value='"._ADD_NEW."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"add\");'>";
					echo "</td>
					<td align='".$defined_right."'>";
						if($this->allowLanguages) echo (($this->allowLanguages) ? get_languages_box("mg_language_id", $arrLanguages[0], "abbreviation", "lang_name", $this->languageId, "", "onchange='javascript:__mgDoPostBack(\"".$this->tableName."\", \"view\", null, null, null, null, \"change_language\", this.value, \"language_id\");'") : "");
					echo "</td>
				</tr>
				<tr><td nowrap height='10px'></td></tr>
			</table>";
		}

		if($this->isFilteringAllowed){
			echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>
				<tr>
					<td align='".$defined_left."'>";
						echo "<b>"._FILTER_BY."</b>: &nbsp;&nbsp;&nbsp;";
						foreach($this->arrFilteringFields as $key => $val){
							$filter_field_value = ($search_status == "active") ? self::GetParameter("filter_by_".$val['table'].$val['field'], false) : "";
							if($val['type'] == "text"){
								echo $key.":&nbsp;<input type='text' class='mgrid_text' name='filter_by_".$val['table'].$val['field']."' value='".$filter_field_value."' style='width:".$val['width'].";' />&nbsp;&nbsp;&nbsp;";
							}else if($val['type'] == "dropdownlist"){
								if(is_array($val['source'])){
									echo $key.":&nbsp;<select class='mgrid_text' name='filter_by_".$val['table'].$val['field']."' style='width:".$val['width'].";'>";
									echo "<option value=''>-- "._SELECT." --</option>";	
									foreach($val['source'] as $key => $val){
										echo "<option ".(($filter_field_value !== "" && $filter_field_value == $key) ? " selected='selected'" : "")." value='".$key."'>".$val."</option>";	
									}
									echo "</select>&nbsp;&nbsp;&nbsp;";
								}								
							}
						}
						if(count($this->arrFilteringFields) > 0){
							echo "&nbsp;";
							if($search_status == "active") echo " <input type='button' class='mgrid_button' name='btnReset' value='"._BUTTON_RESET."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"view\", \"\", \"\", \"\", \"\", \"reset_filtering\");'>";
							echo " <input type='button' class='mgrid_button' name='btnSearch' value='"._SEARCH."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"view\", \"\", \"\", \"\", \"\", \"filtering\");'>";
						}
			echo "	</td>
				</tr>
				<tr><td nowrap height='10px'></td></tr>
			</table>";
		}
		
		// draw rows
		if($arrRecords[1] > 0){			
			echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>";
			// draw column headers
			echo "<tr>";
				foreach($this->arrViewModeFields as $key => $val){
					$width = isset($val['width']) ? "width='".$val['width']."'": "";
					$align = isset($val['align']) ? "align='".$val['align']."'": "";
					$visible = isset($val['visible']) ? $val['visible'] : true;
					if($visible){
						echo "<th ".$width." ".$align.">";
							if($this->isSortingAllowed){
								echo "<a href='javascript:void(0);' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"view\", \"\", \"".$key."\", \"".$sorting_types."\", \"".$page."\", \"sorting\");'><b>".$val['title']."</b></a>";
							}else{
								echo $val['title'];
							}
						echo "</th>";
					}					
				}				
			if($this->actions['details'] || $this->actions['edit'] || $this->actions['delete']){
				echo "<th width='10%'>"._ACTIONS."</th>";
			}
			echo "</tr>";
			echo "<tr><td colspan='".$colspan."' height='3px' nowrap='nowrap'>".draw_line("no_margin_line", IMAGE_DIRECTORY, false)."</td></tr>";
			for($i=0; $i<$arrRecords[1]; $i++){
				echo "<tr ".(($this->isAlterColorsAllowed) ? highlight(0) : "")." onmouseover='oldColor=this.style.backgroundColor;this.style.backgroundColor=\"#e7e7e7\";' onmouseout='this.style.backgroundColor=oldColor'>";
					foreach($this->arrViewModeFields as $key => $val){
						$align = isset($val['align']) ? "align='".$val['align']."'": "";
						if(isset($arrRecords[0][$i][$key])){
							$field_value = $this->DrawFieldByType("view", $key, $val, $arrRecords[0][$i], false);								
						}else{
							if($this->debug) $this->arrWarnings['wrong_'.$key] = "Field <b>".$key."</b>: wrong definition in View mode or field has no value in SQL! Please check currefully your code.";
							$field_value = "";
						}
						$visible = isset($val['visible']) ? $val['visible'] : true;
						if($visible){
							echo "<td ".$align.">".$field_value."</td>";							
						}
					}				
					if($this->actions['details'] || $this->actions['edit'] || $this->actions['delete']){
						echo "<td align='center' nowrap>";
						if($this->actions['details']){
							echo "<a href='javascript:void(0);' title='"._VIEW_WORD."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"details\", \"".$arrRecords[0][$i]['id']."\");'>".(($this->actionIcons) ? "<img src='images/microgrid_icons/details.gif' title='"._VIEW_WORD."' alt='' border='0' style='margin:0px; padding:0px; ' height='16px' />" : _VIEW_WORD)."</a>";
						}				
						if($this->actions['edit'])   {
							if($this->actions['details']) echo "&nbsp;".get_divider()."&nbsp";  
							echo "<a href='javascript:void(0);' title='"._EDIT_WORD."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"edit\", \"".$arrRecords[0][$i]['id']."\");'>".(($this->actionIcons) ? "<img src='images/microgrid_icons/edit.gif' title='"._EDIT_WORD."' alt='' border='0' style='margin:0px; padding:0px;' height='16px' />" : _EDIT_WORD)."</a>";
						}
						if($this->actions['delete']) {
							if($this->actions['edit'] || $this->actions['details']) echo "&nbsp;".get_divider()."&nbsp"; 
							echo "<a href='javascript:void(0);' title='"._DELETE_WORD."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"delete\", \"".$arrRecords[0][$i]['id']."\");'>".(($this->actionIcons) ? "<img src='images/microgrid_icons/delete.gif' title='"._DELETE_WORD."' alt='' border='0' style='margin:0px; padding:0px;' height='16px' />" : _DELETE_WORD)."</a>";
						}
						echo "&nbsp;</td>";
					}
				echo "</tr>";
			}

			echo "<tr><td colspan='".$colspan."' height='15px' nowrap='nowrap'>".draw_line("no_margin_line", IMAGE_DIRECTORY, false)."</td></tr>";
			echo "</table>";
			
			echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>";
			echo "<tr>";
			echo "	<td>";
				if($this->isPagingAllowed){
					echo "<b>"._PAGES.":</b> ";
					for($i = 1; $i <= $total_pages; $i++){
						echo "<a class='pagging_link' href='javascript:void(0);' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"view\", \"\", \"\", \"\", \"".$i."\", \"\");'>".(($i == $page) ? "<b>[".$i."]</b>" : $i)."</a> ";
					}				
				}			
			echo "	</td>";
			echo "	<td align='right'><b>"._TOTAL."</b>: ".$total_records."</td>";
			echo "</tr>";
			echo "</table>";
		}else{
			echo draw_message(_NO_RECORDS_FOUND, true, true);
			//if($this->debug) $this->arrSQLs['select'] = $this->VIEW_MODE_SQL." ".$this->WHERE_CLAUSE." ".$this->ORDER_CLAUSE." LIMIT ".$start_row.", ".(int)$this->pageSize;
		}
		
		echo "</form>";
		
		$this->DrawRunningTime();
		$this->DrawErrors();
		$this->DrawWarnings();
		$this->DrawSQLs();	
		$this->DrawPostInfo();	
	}

	/***********************************************************************
	 *
	 *	Draw Add Mode
	 *	
	 ***********************************************************************/
	function DrawAddMode(){
		
		$this->IncludeJSFunctions();
		
		$sorting_fields  = self::GetParameter('sorting_fields');
		$sorting_types   = self::GetParameter('sorting_types');
		$page 			 = self::GetParameter('page');
		$operation 		 = self::GetParameter('operation');
		$operation_type  = self::GetParameter('operation_type');
		$operation_field = self::GetParameter('operation_field');
		$search_status   = self::GetParameter('search_status');

		echo "<form name='frmMicroGrid_".$this->tableName."' id='frmMicroGrid_".$this->tableName."' action='".$this->formActionURL."' method='post' enctype='multipart/form-data'>\n";
		echo draw_hidden_field($this->uPrefix."mg_action", "create")."\n";
		echo draw_hidden_field("mg_rid", "-1")."\n";
		echo draw_hidden_field("mg_sorting_fields", $sorting_fields)."\n";
		echo draw_hidden_field("mg_sorting_types", $sorting_types)."\n";
		echo draw_hidden_field("mg_page", $page)."\n";
		echo draw_hidden_field("mg_operation", $operation)."\n";
		echo draw_hidden_field("mg_operation_type", $operation_type)."\n";
		echo draw_hidden_field("mg_operation_field", $operation_field)."\n";
		echo draw_hidden_field("mg_search_status", $search_status)."\n";
		echo draw_hidden_field("mg_language_id", $this->languageId)."\n";

		// draw hidden fields
		foreach($this->arrAddModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info"){						
						if($v_val['type'] == "hidden"){
							draw_hidden_field($v_key, $v_val['default'])."\n";
						}				
					}
				}				
			}else{
				if($val['type'] == "hidden"){
					draw_hidden_field($key, $val['default'])."\n";
				}				
			}
		}				
		
		// draw Add Form
		echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";		
		echo "<tr><td colspan='2' height='1px' nowrap></td></tr>\n";
		foreach($this->arrAddModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				echo "</table><br />\n";
				echo "<fieldset style='padding:5px;margin-left:5px;margin-right:10px;'>";
				echo isset($val['separator_info']['legend']) ? "<legend>".$val['separator_info']['legend']."</legend>" : "";
				echo "\n<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";		
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info" && $v_val['type'] != "hidden"){						
						echo "<tr>\n";
						echo "  <td width='25%'>".$v_val['title'].((isset($v_val['required']) && $v_val['required']) ? " <font color='#c13a3a'>*</font>" : "").":</td>\n";
						echo "  <td style='padding-left:6px;'>".$this->DrawFieldByType("add", $v_key, $v_val, $this->params, false)."</td>\n";
						echo "</tr>\n";
					}
				}
				echo "</table>\n";
				echo "</fieldset>\n";				
				echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";		
			}else{
				if($val['type'] != "hidden"){
					echo "<tr>\n";
					echo "  <td width='25%'>".$val['title'].((isset($val['required']) && $val['required']) ? " <font color='#c13a3a'>*</font>" : "").":</td>\n";
					echo "  <td>".$this->DrawFieldByType("add", $key, $val, $this->params, false)."</td>\n";
					echo "</tr>\n";				
				}							
			}
		}				
		echo "<tr><td colspan='2' height='15px' nowrap></td></tr>\n";
		echo "<tr>\n
				<td colspan='2'>
					<input class='mgrid_button' type='button' name='subAddNewRecord' value='"._BUTTON_CREATE."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"create\");'>
					<input class='mgrid_button' type='button' name='btnCancel' value='"._BUTTON_CANCEL."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"view\");'>
				</td>
			  <tr>\n";		
		echo "</table><br />\n";
		echo "</form>\n";

		if($this->errorField != "") echo "<script type='text/javascript'>__mgSetFocus('".$this->errorField."');</script>";
			
		$this->DrawRunningTime();
		$this->DrawErrors();
		$this->DrawWarnings();
		$this->DrawSQLs();	
		$this->DrawPostInfo();	
	}

	
	/***********************************************************************
	 *
	 *	Draw Edit Mode
	 *	
	 ***********************************************************************/
	function DrawEditMode($rid = "0", $operation = "", $operation_field = "", $buttons = array("reset"=>false, "cancel"=>true)){
		
		$this->IncludeJSFunctions();
		
		$sorting_fields  = self::GetParameter('sorting_fields');
		$sorting_types   = self::GetParameter('sorting_types');
		$page 			 = self::GetParameter('page');
		$operation 		 = self::GetParameter('operation');
		$operation_type  = self::GetParameter('operation_type');
		$operation_field = self::GetParameter('operation_field');
		$search_status   = self::GetParameter('search_status');

		echo "\n<form name='frmMicroGrid_".$this->tableName."' id='frmMicroGrid_".$this->tableName."' action='".$this->formActionURL."' method='post' enctype='multipart/form-data'>\n";
		echo draw_hidden_field($this->uPrefix."mg_action", "update")."\n";
		echo draw_hidden_field("mg_rid", $rid)."\n";
		echo draw_hidden_field("mg_sorting_fields", $sorting_fields)."\n";
		echo draw_hidden_field("mg_sorting_types", $sorting_types)."\n";
		echo draw_hidden_field("mg_page", $page)."\n";
		echo draw_hidden_field("mg_operation", "")."\n";
		echo draw_hidden_field("mg_operation_type", "")."\n";
		echo draw_hidden_field("mg_operation_field", "")."\n";
		echo draw_hidden_field("mg_search_status", $search_status)."\n";
		echo draw_hidden_field("mg_language_id", $this->languageId)."\n";
		// save filter (search) data for view mode
		if($this->isFilteringAllowed){
			foreach($this->arrFilteringFields as $key => $val){
				//if($val['type'] == "text"){
					$filter_field_value = ($search_status == "active") ? self::GetParameter("filter_by_".$val['table'].$val['field'], false) : "";
					echo draw_hidden_field("filter_by_".$val['table'].$val['field'], $filter_field_value)."\n";
				//}
			}
		}

		// 1. prepare password fields
		foreach($this->arrEditModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info"){						
						// prepare password
						if($v_val['type'] == "password"){
							$password_field = $this->UncryptPasswordValue($v_key, $v_val);
							$this->EDIT_MODE_SQL = str_replace($this->tableName.".".$v_key, $password_field, $this->EDIT_MODE_SQL);
						}											
					}					
				}				
			}else{
				// prepare password
				if($val['type'] == "password"){
					$password_field = $this->UncryptPasswordValue($key, $val);
					$this->EDIT_MODE_SQL = str_replace($this->tableName.".".$key, $password_field, $this->EDIT_MODE_SQL);
				}
			}
		}						

		$this->EDIT_MODE_SQL = str_replace("_RID_", $rid, $this->EDIT_MODE_SQL);
		$result = database_query($this->EDIT_MODE_SQL, DATA_AND_ROWS);		
		if($this->debug) $this->arrSQLs['select_edit_mode'] = $this->EDIT_MODE_SQL;
		
		// 1. draw hidden fields
		// 2. delete files/images
		foreach($this->arrEditModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info"){						
						// delete file/image
						if($operation == "remove" && $operation_field != "" && ($v_key == $operation_field)){
							$this->RemoveFileImage($rid, $operation_field, $v_val['target'], $result[0][0][$v_key]);
							$result[0][0][$v_key] = "";
						}
						// draw hidden field
						if($v_val['type'] == "hidden"){
							draw_hidden_field($v_key, $result[0][0][$v_key]);
							echo "\n";
						}
					}					
				}				
			}else{
				// delete file/image
				if($operation == "remove" && $operation_field != "" && ($key == $operation_field)){
					$this->RemoveFileImage($rid, $operation_field, $val['target'], $result[0][0][$key]);
					$result[0][0][$key] = "";
				}
				// draw hidden field
				if($val['type'] == "hidden"){
					draw_hidden_field($key, $result[0][0][$key]);
					echo "\n";
				}
			}
		}								

		// draw Edit Form
		echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";
		$blocks_count = "0";
		foreach($this->arrEditModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				echo "</table>".(($blocks_count++ > 0) ? "<br />" : "")."\n";
				echo "<fieldset style='padding:5px;margin-left:5px;margin-right:10px;'>";
				echo "<legend>".$val['separator_info']['legend']."</legend>";
				echo "\n<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";		
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info" && $v_val['type'] != "hidden"){
						echo "<tr>\n";
						echo "  <td width='25%'>".$v_val['title'].((isset($v_val['required']) && $v_val['required']) ? " <font color='#c13a3a'>*</font>" : "").":</td>\n";
						if(isset($this->params[$v_key]) && $this->params[$v_key] !== "" && ($v_val['type'] != "checkbox")){
							echo "  <td style='padding-left:6px;'>".$this->DrawFieldByType("edit", $v_key, $v_val, $this->params, false)."</td>\n";
						}else{
							echo "  <td style='padding-left:6px;'>".$this->DrawFieldByType("edit", $v_key, $v_val ,$result[0][0], false)."</td>\n";
						}
						echo "</tr>\n";
					}
				}
				echo "</table>\n";
				echo "</fieldset>\n";				
				echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";		
			}else{
				if($val['type'] != "hidden"){
					echo "<tr>\n";
					echo "  <td width='25%'>".$val['title'].((isset($val['required']) && $val['required']) ? " <font color='#c13a3a'>*</font>" : "").":</td>\n";
					if(isset($this->params[$key]) && $this->params[$key] !== "" && ($val['type'] != "checkbox")){
						echo "  <td>".$this->DrawFieldByType("edit", $key, $val, $this->params, false)."</td>\n";													
					}else{
						echo "  <td>".$this->DrawFieldByType("edit", $key, $val, $result[0][0], false)."</td>\n";						
					}
					echo "</tr>\n";				
				}
			}
		}
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>";
		echo "<tr>
				<td colspan='2'>
					<input class='mgrid_button' type='submit' name='subUpdateRecord' value='"._BUTTON_UPDATE."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"update\");' />
					".(($buttons["reset"]) ? "<input class='mgrid_button' type='reset' name='btnReset' value='"._BUTTON_RESET."' / >" : "")."
					".(($buttons["cancel"]) ? "<input class='mgrid_button' type='button' name='btnCancel' value='"._BUTTON_CANCEL."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"view\");' />" : "")."					
				</td>
			  <tr>\n";		
		echo "</table><br />\n";
		echo "</form>\n";

		if($this->errorField != "") echo "<script type='text/javascript'>__mgSetFocus('".$this->errorField."');</script>";	   
		
		$this->DrawRunningTime();
		$this->DrawErrors();
		$this->DrawWarnings();
		$this->DrawSQLs();	
		$this->DrawPostInfo();	
	}


	
	/***********************************************************************
	 *
	 *	Draw Details Mode
	 *	
	 ***********************************************************************/
	function DrawDetailsMode($rid = "0"){
		
		$this->IncludeJSFunctions();
		
		$sorting_fields  = self::GetParameter('sorting_fields');
		$sorting_types   = self::GetParameter('sorting_types');
		$page 			 = self::GetParameter('page');
		$operation 		 = self::GetParameter('operation');
		$operation_type  = self::GetParameter('operation_type');
		$operation_field = self::GetParameter('operation_field');
		$search_status   = self::GetParameter('search_status');

		echo "\n<form name='frmMicroGrid_".$this->tableName."' id='frmMicroGrid_".$this->tableName."' action='".$this->formActionURL."' method='post' enctype='multipart/form-data'>\n";
		echo draw_hidden_field($this->uPrefix."mg_action", "details")."\n";
		echo draw_hidden_field("mg_rid", $rid)."\n";
		echo draw_hidden_field("mg_sorting_fields", $sorting_fields)."\n";
		echo draw_hidden_field("mg_sorting_types", $sorting_types)."\n";
		echo draw_hidden_field("mg_page", $page)."\n";
		// to prevent re-sorting on back to view mode $operation = ""
		echo draw_hidden_field("mg_operation", "")."\n";
		echo draw_hidden_field("mg_operation_type", $operation_type)."\n";
		echo draw_hidden_field("mg_operation_field", $operation_field)."\n";
		echo draw_hidden_field("mg_search_status", $search_status)."\n";
		echo draw_hidden_field("mg_language_id", $this->languageId)."\n";
		// save filter (search) data for view mode
		if($this->isFilteringAllowed){
			foreach($this->arrFilteringFields as $key => $val){
				//if($val['type'] == "text"){
					$filter_field_value = ($search_status == "active") ? self::GetParameter("filter_by_".$val['table'].$val['field'], false) : "";
					echo draw_hidden_field("filter_by_".$val['table'].$val['field'], $filter_field_value)."\n";
				//}
			}
		}

		// ger result for edited row		
		$this->DETAILS_MODE_SQL = str_replace("_RID_", $rid, $this->DETAILS_MODE_SQL);
		$result = database_query($this->DETAILS_MODE_SQL, DATA_AND_ROWS);	
		if($this->debug) $this->arrSQLs['select_details'] = $this->DETAILS_MODE_SQL;
	
		// draw Details Form
		echo "<table width='100%' border='0' cellspacing='2' cellpadding='2' class='mgrid_table'>\n";
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>";
		foreach($this->arrDetailsModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				echo "</table><br />\n";
				echo "<fieldset style='padding:5px;margin-left:5px;margin-right:10px;'>";
				echo "<legend>".$val['separator_info']['legend']."</legend>";
				echo "\n<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";		
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info"){						
						echo "<tr>\n";
						echo "  <td width='27%'>".$v_val['title'].":</td>\n";
						echo "  <td style='padding-left:6px;'>".$this->DrawFieldByType("details", $v_key, $v_val, $result[0][0], false)."</td>\n";
						echo "</tr>\n";
					}
				}
				echo "</table>\n";
				echo "</fieldset>\n";				
				echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";		
			}else{
				if($val['type'] != "hidden"){
					echo "<tr>\n";
					echo "  <td width='25%'>".ucfirst($val['title']).":</td>\n";
					echo "  <td>".$this->DrawFieldByType("details", $key, $val, $result[0][0], false)."</td>\n";
					echo "</tr>\n";				
				}			
			}
		}
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>";
		echo "<tr>
				<td colspan='2'>
					<input class='mgrid_button' type='button' name='btnBack' value='"._BUTTON_BACK."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"view\");'> 
				</td>
			  <tr>\n";		
		echo "</table><br />\n";
		echo "</form>\n";

		$this->DrawRunningTime();
		$this->DrawErrors();
		$this->DrawWarnings();
		$this->DrawSQLs();	
		$this->DrawPostInfo();	
	}


	/***
	 * Draw field by type
	 *		@param $field_name
	 *		@param $field_array - ['field'] => array("".....)
	 **/	
	function DrawFieldByType($mode, $field_name, $field_array = array(), $params = array(), $draw = true){
		$output = "";
		
		if($field_name == "") return false;

		//print_r($field_array);

		$rid 		  = isset($params[$this->primaryKey]) ? $params[$this->primaryKey] : "";
		$field_type   = isset($field_array['type']) ? $field_array['type'] : "";
		$source       = isset($field_array['source']) ? $field_array['source'] : "";
		$readonly     = (isset($field_array['readonly']) && $field_array['readonly'] === true) ? true : false;
		$default 	  = isset($field_array['default']) ? $field_array['default'] : "";
		$true_value   = isset($field_array['true_value']) ? $field_array['true_value'] : "1";
		$width        = isset($field_array['width']) ? $field_array['width'] : "";
		$height       = isset($field_array['height']) ? $field_array['height'] : "";
		$image_width  = isset($field_array['image_width']) ? $field_array['image_width'] : "";
		$image_height = isset($field_array['image_height']) ? $field_array['image_height'] : "";
		$target       = isset($field_array['target']) ? $field_array['target'] : "";
		$maxlength    = isset($field_array['maxlength']) ? $field_array['maxlength'] : "";
		$no_image     = isset($field_array['no_image']) ? $field_array['no_image'] : "";
		$required 	  = isset($field_array['required']) ? $field_array['required'] : false;
		$pre_html 	  = isset($field_array['pre_html']) ? $field_array['pre_html'] : "";
		$post_html 	  = isset($field_array['post_html']) ? $field_array['post_html'] : "";
		$formate 	  = isset($field_array['formate']) ? $field_array['formate'] : "";
		
		$atr_readonly = ($readonly) ? " readonly='readonly'" : "";
		$atr_disabled = ($readonly) ? " disabled='disabled'" : "";
		$css_disabled = ($readonly) ? " mgrid_disabled" : "";
		$attr_maxlength = ($maxlength != "") ? " maxlength='".intval($maxlength)."'" : "";

		$field_value  = isset($params[$field_name]) ? $params[$field_name] : "";
		if($mode == "add" && $field_value == "") $field_value = $default;
		if($this->isHtmlEncoding) $field_value = $this->GetDataDecoded($field_value);

		if($mode == "view"){
			// View Mode
			switch($field_type){
				case "image":
				    if($field_value == "" && $no_image != "") $field_value = $no_image;
					$output = "<img src='".$target.$field_value."' title='".$field_value."' alt='' width='".$image_width."' height='".$image_height."' />";				
					break;
				
				default:
				case "label":
					$title = "";
					if($formate == "strip_tags"){
						$field_value = strip_tags($field_value);
					}
					if($maxlength != "" && $this->IsInteger($maxlength)){
						if(strlen($field_value) > $maxlength){
							$title = $field_value;
							$field_value = substr($field_value, 0, $maxlength)."...";						
						}
					}
					$output = "<label class='mgrid_label' title='".$this->GetDataDecoded($title)."'>".$field_value."</label>";			
					break;			
			}			
		}else{
			// Add/Edit/Detail Modes 
			switch($field_type){
				case "checkbox":
					$checked = "";				
					if($mode == "add" && $default == $true_value){
						$checked = " checked='checked'";
					}else{
						if($field_value == "1") $checked = " checked='checked'";
					}
					if($readonly){
						$output  = "<input type='checkbox' name='".$field_name."' id='".$field_name."' class='mgrid_checkbox' value='1'".$checked.$atr_disabled." />";						
						$output .= "<input type='hidden' name='".$field_name."' id='".$field_name."' value='1' />";
					}else{
						$output = "<input type='checkbox' name='".$field_name."' id='".$field_name."' class='mgrid_checkbox' value='1'".$checked." />";						
					}
					break;			
	
				case "file":
				case "image":
					if(strtolower(SITE_MODE) == "demo") $atr_readonly = " disabled='disabled'";
				
					if(($mode == "edit" || $mode == "details")){
						//if(file_exists($target.$field_value)){						
						//}
						if($mode == "edit"){
							if($field_value != ""){
								$output = "<img src='".$target.$field_value."' title='".$field_value."' alt='' width='120px' height='90px' />";								
								if($required) $output .= "<input type='hidden' name='".$field_name."' id='".$field_name."' value='".$field_value."' />";
								$output .= "<br /><a href='".$this->formActionURL."&".$this->uPrefix."mg_action=edit&mg_rid=".$rid."&mg_operation=remove&mg_operation_field=".$field_name."'>["._DELETE_WORD."]</a>";
							}else{
								$output = "<input type='file' name='".$field_name."' id='".$field_name."' class='mgrid_file' ".$atr_readonly."/>";		
							}
						}else if($mode == "details"){
							if($field_value == "" && $no_image != "") $field_value = $no_image;
							$output = "<img src='".$target.$field_value."' title='".$field_value."' alt='' width='120px' height='90px' />";
						}
					}else{
						$output = "<input type='file' name='".$field_name."' id='".$field_name."' class='mgrid_file' ".$atr_readonly."/>";
					}
					break;
				
				case "enum":
					if(is_array($source)){				    
						$output_start = "<select class='mgrid_select' name='".$field_name."'
												id='".$field_name."'
												style='".(($width!="")?"width:".$width.";":"")."'
												".(($readonly) ? "disabled='disabled'" : "").">";
						$output_options = "<option value=''>-- "._SELECT." --</option>";
						foreach($source as $key => $val){
							$output_options .= "<option value='".$key. "' ";
							$output_options .= ($field_value == $key) ? "selected='selected' " : "";
							$output_options .= ">".$val."</option>";
						}
						$output = $output_start.$output_options."</select>";					
					}
					break;			
				
				case "label":
					$title = "";
					if($maxlength != "" && $this->IsInteger($maxlength)){
						if(strlen($field_value) > $maxlength){
							$title = $field_value;
							$field_value = substr($field_value, 0, 18)."...";						
						}
					}
					$output = "<label class='mgrid_label' title='".$title."'>".$this->GetDataDecoded($field_value)."</label>";			
					break;			
	
				case "password":
					if($mode == "add" || $mode == "edit"){
						$output = "<input class='mgrid_text' name='".$field_name."' id='".$field_name."'  style='".(($width!="")?"width:".$width.";":"")."' value='".$field_value."' ".$atr_readonly.$attr_maxlength." />";
					}else{
						$output = "<label class='mgrid_label'>*****</label>";				
					}				
					break;			
	
				case "textarea":
					$output = $pre_html."<textarea class='mgrid_textarea' name='".$field_name."' id='".$field_name."' style='".(($width!="")?"width:".$width.";":" rows='7'").(($height!="")?"height:".$height.";":" cols='60'")."' ".$atr_disabled.">".$this->GetDataDecodedText($field_value)."</textarea>".$post_html;
					break;			
	
				default:
				case "textbox":
					$output = $pre_html."<input class='mgrid_text".$css_disabled."' name='".$field_name."' id='".$field_name."'  style='".(($width!="")?"width:".$width.";":"")."' value='".$this->GetDataDecoded($field_value)."' ".$atr_readonly.$attr_maxlength." />".$post_html;
					break;			
				
			}			
		}		
		
		if($draw) echo $output;
		else return $output;		
	}

	
	/***
	 * Get data encoded text
	 *		@param $string
	 **/
	private function GetDataEncodedText($string = ""){
		return $string;
	}
	
	
	/***
	 * Get data decoded text
	 *		@param $string
	 **/
	private function GetDataDecodedText($string = ""){
		$string = str_replace('\\\\', "\\", $string); // double quotation mark
		$string = str_replace('\"', '"', $string); // double quotation mark
		$string = str_replace("\'", "'", $string); // single quotation mark                        
		return $string;
	}
	
	
	/***
	 * Get data encoded
	 *		@param $string
	 **/
	private function GetDataEncoded($string = ""){
		$search	 = array("\\","\0","\n","\r","\x1a","'",'"');
		$replace = array("\\\\","\\0","\\n","\\r","\Z","\'",'\"');
		return str_replace($search, $replace, $string);
	}
	
	
	/***
	 * Get data decoded 
	 *		@param $string
	 **/
	private function GetDataDecoded($string = ""){
		$search  = array("\\\\","\\0","\\n","\\r","\Z","\'",'\"','"',"'");
		$replace = array("\\","\0","\n","\r","\x1a","&#039;",'&#034;',"&#034;","&#039;");
		return str_replace($search, $replace, $string);
	}

	////////////////////////////////////////////////////////////////////////////
	// Validation methods
	/***
	 * Fields validation procedure
	 * 		@param $array - validation array of fields
	 **/
	private function FieldsValidation($array)
	{		
		if(!is_array($array)) return false;
		
		foreach($array as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info"){						
						if(!$this->ValidateField($v_key, $v_val)){
							$this->errorField = $v_key;
							return false;
						}						
					}
				}
			}else{
				if(!$this->ValidateField($key, $val)){
					$this->errorField = $key;
					return false;
				}										
			}
		}
		return true;
	}
	
	/***
	 * Validate field
	 **/
	private function ValidateField($key, $val)
	{
		$validation_type = isset($val['validation_type']) ? strtolower($val['validation_type']) : "";
		$validation_type_parts = explode("|", $validation_type);
		$validation_type = isset($validation_type_parts[0]) ? $validation_type_parts[0] : "";
		$validation_sub_type = isset($validation_type_parts[1]) ? $validation_type_parts[1] : "";

		$validation_maxlength = (isset($val['validation_maxlength']) && $this->IsInteger($val['validation_maxlength'])) ? $val['validation_maxlength'] : "";
		
		$field_type = isset($val['type']) ? strtolower($val['type']) : "";
		$required = isset($val['required']) ? $val['required'] : false;
		$readonly = isset($val['readonly']) ? $val['readonly'] : false;
		
		// check image fields
		if($field_type == "image" || $field_type == "file"){
			$field_value = isset($_FILES[$key]['name']) ? $_FILES[$key]['name'] : "";
			if($field_value == "") $field_value = isset($_POST['image_file']) ? $_POST['image_file'] : "";
			if($required && !$field_value){
				$this->error = str_replace("_FIELD_", $val['title'], _FIELD_CANNOT_BE_EMPTY);
				return false;
			}
			return true;
		}
		
		if($required && isset($this->params[$key]) && $this->params[$key] == "" && !$readonly){ // 
			$this->error = str_replace("_FIELD_", $val['title'], _FIELD_CANNOT_BE_EMPTY);
			return false;
		}		

		if(isset($this->params[$key]) && $this->params[$key] != ""){
			if($validation_type == "email" && !$this->IsEmail($this->params[$key])){
				$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_EMAIL);
				return false;					
			}else if($validation_type == "alpha" && !$this->IsAlpha($this->params[$key])){
				$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_ALPHA);
				return false;					
			}else if($validation_type == "numeric"){
				if(!$this->IsNumeric($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_NUMERIC);
					return false;
				}else if($validation_sub_type == "positive" && $this->params[$key] < 0){
					$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_NUMERIC_POSITIVE);
					return false;					
				}
			}else if($validation_type == "float"){
				if(!$this->IsFloat($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_FLOAT);
					return false;
				}else if($validation_sub_type == "positive" && $this->params[$key] < 0){
					$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_FLOAT_POSITIVE);
					return false;
				}				
			}else if($validation_type == "alpha_numeric" && !$this->IsAlphaNumeric($this->params[$key])){
				$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_ALPHA_NUMERIC);
				return false;					
			}else if($validation_type == "text" && !$this->IsText($this->params[$key])){
				$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_TEXT);
				return false;					
			}else if($validation_type == "password" && !$this->IsPassword($this->params[$key])){
				$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_PASSWORD);
				return false;					
			}else if($validation_type == "ip_address" && !$this->IsIpAddress($this->params[$key])){
				$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_IP_ADDRESS);
				return false;					
			}
			
			if($validation_maxlength > 0 && strlen($this->params[$key]) > $validation_maxlength){
				$this->error = str_replace("_FIELD_", $val['title'], _FIELD_LENGTH_EXCEEDED);
				$this->error = str_replace("_SIZE_", $validation_maxlength, $this->error);
				return false;									
			}
		}
		
		return true;
	}
	
	
	/***
	 * Email Validation
	 **/
	protected function IsEmail($field = ""){
		$strict = false;
		$regex = $strict ? '/^([.0-9a-z_-]+)@(([0-9a-z-]+\.)+[0-9a-z]{2,4})$/i' :  '/^([*+!.&#$\'\\%\/0-9a-z^_`{}=?~:-]+)@(([0-9a-z-]+\.)+[0-9a-z]{2,4})$/i';
		
		if(preg_match($regex, trim($field))) {
		   return true;
		} else {
		   return false;
		}		
	}

	/***
	 * Numeric Validation
	 **/
	protected function IsNumeric($field = ""){
		return ($field == strval(intval($field))) ? true : false;
	}

	/***
	 * Float Number Validation
	 **/
	protected function IsFloat($field = ""){
		return ($field == strval(floatval($field))) ? true : false;
	}

	/***
	 * Alpha Validation
	 **/
	protected function IsAlpha($field = ""){
		if(ctype_alpha($field)){
		   return true;
		} else {
		   return false;
		}		
	}

	/***
	 * Alpha Numeric Validation
	 **/
	protected function IsAlphaNumeric($field = ""){
		if(ctype_alnum($field)){
		   return true;
		} else {
		   return false;
		}		
	}

	/***
	 * Text Validation
	 **/
	protected function IsText($field = ""){
		if(ctype_alnum($field) || $field != ""){
		   return true;
		} else {
		   return false;
		}		
	}

	/***
	 * Password Validation
	 **/
	protected function IsPassword($field = ""){
		if(ctype_alnum($field) && $field != "" && strlen($field) >= 6){
		   return true;
		} else {
		   return false;
		}		
	}
	
	/***
	 * IP Address Validation
	 **/
	protected function IsIpAddress($field = ""){
		// format of the ip address is matched
		if(preg_match("/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/",$field)){
			// all the intger values are separated
			$parts=explode(".",$field);
			// check each part can range from 0-255
			foreach($parts as $ip_parts){
				//if number is not within range of 0-255
				if(intval($ip_parts)>255 || intval($ip_parts)<0) return false; 				
			}
			return true;
		}else{
			return false;
		}
	}

	/***
	 * Integer Validation
	 **/
	protected function IsInteger($field = ""){
        if(is_numeric($field) === true){
            if((int)$field == $field){
                return true;
            }else{
                return false;
            }
        }else{
            return false;
        }
    }
	
	
	/***
	 * Crypt password value
	 **/
	private function CryptPasswordValue($key, $field_array, $value){
		$cryptography = $field_array['cryptography'];
		$cryptography_type = strtolower($field_array['cryptography_type']);
		$aes_password = $field_array['aes_password'];
        if($cryptography === true || strtolower($cryptography) == "true"){
            if($cryptography_type == "md5"){
                return "MD5('".$value."')";
            }else if($cryptography_type == "aes"){
                return "AES_ENCRYPT('".$value."', '".$aes_password."')";                
            }
        }
        return "'".$value."'";    
	}
	
	/***
	 * Uncrypt password value
	 **/
	private function UncryptPasswordValue($key, $field_array){
		$output = $key;
		$cryptography = $field_array['cryptography'];
		$cryptography_type = strtolower($field_array['cryptography_type']);
		$aes_password = $field_array['aes_password'];
		if($cryptography === true || strtolower($cryptography) == "true"){
			if($cryptography_type == "aes"){
				$output = "AES_DECRYPT(".$key.", '".$aes_password."') as ".$key;    
			}
		}
        return $output;    
	}
	
	/***
	 * Include JavaScript
	 **/
	protected function IncludeJSFunctions(){
		echo "<script type='text/javascript' src='include/classes/js/microgrid.js'></script>\n";
		echo "<script type='text/javascript' src='include/classes/js/lang/en.js'></script>\n";
	}
	
	/**
	 *	Draw system errors
	 *
	*/	
	protected function DrawErrors($draw = true){
		if(!$this->debug) return false;
		$output = "<br><div style='width:100%; text-align:left; color:#860000;'>";
		$output .= "Errors(".count($this->arrErrors)."):<br />------<br />";
		if(count($this->arrErrors) > 0){
			foreach($this->arrErrors as $key){
				$output .= "<span>* ".$key."</span><br />";
			}
		}
		$output .= "</div>";
		if($draw) echo $output;
		else return $output;		
	}
	
	/**
	 *	Draw system warnings
	 *
	*/	
	protected function DrawWarnings($draw = true){
		if(!$this->debug) return false;
		$output = "<br><div style='width:100%; text-align:left; color:#cc9900;'>";
		$output .= "Warnings(".count($this->arrWarnings)."):<br />------<br />";
		if(count($this->arrWarnings) > 0){
			foreach($this->arrWarnings as $key){
				$output .= "<span>* ".$key."</span><br />";
			}
		}
		$output .= "</div>";
		if($draw) echo $output;
		else return $output;		
	}
	
	/**
	 *	Set system SQLs
	 *
	*/	
	protected function SetSQLs($key, $msg){
		if($this->debug) $this->arrSQLs[$key] = $msg;					
	}

	/**
	 *	Draw system SQLs
	 *
	*/	
	protected function DrawSQLs($draw = true){
		if(!$this->debug) return false;
		$output = "<br><div style='width:100%; text-align:left; color:#444444;'>";
		$output .= "SQL:<br />------<br />";
		if(count($this->arrSQLs) > 0){
			$output .= "<ol>";
			foreach($this->arrSQLs as $key){
				$output .= "<li>".htmlentities($key)."</li><br />";
			}
			$output .= "</ol>";
		}
		$output .= "</div>";
		if($draw) echo $output;
		else return $output;		
	}

	/**
	 *	POST data
	*/	
	protected function DrawPostInfo($draw = true){
		if(!$this->debug) return false;
		$output = "<br><div style='width:100%; text-align:left; color:#008600;'>";
		$output .= "POST:<br />------<br /><pre>";
		$output .= print_r($_POST, true);
		$output .= "</pre>";
		$output .= "</div>";
		if($draw) echo $output;
		else return $output;		
	}
	
	/**
	 *	Upload image/file 
	*/	
	private function UploadFileImage($mode){
		$arrUploadedFiles = array();
		
		$system = (stristr(PHP_OS, 'WIN')) ? "windows" : "linux";

		#if ($uploaded_size > 350000){echo "Your file is too large.<br>"; $ok=0;} 				
		#if ($uploaded_type =="text/php"){echo "No PHP files<br>";$ok=0;} 
		#if (!($uploaded_type=="image/gif")) {echo "You may only upload GIF files.<br>";$ok=0;} 				

		if($mode == "add"){
			$arrModeFields = & $this->arrAddModeFields;
		}else if($mode == "edit"){
			$arrModeFields = & $this->arrEditModeFields;
		}
		
		foreach($arrModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info"){						
						if(isset($v_val['type']) && ($v_val['type'] == "image" || $v_val['type'] == "file")){
							if(isset($_FILES[$v_key]['name'])){
								if($system == "windows"){
									$target = str_replace("index.php", "", $_SERVER['SCRIPT_FILENAME']).$v_val['target'];
									$target = str_replace("/", "\\", $target);
									$target = str_replace("\\", "\\\\", $target);
								}else{
									$target = $v_val['target'];
								}								
                                $random_name = isset($v_val['random_name']) ? $v_val['random_name'] : false; 
								if($random_name == "true" || $random_name === true){									
									$target_file_name = basename($_FILES[$v_key]['name']);
									$ext = substr(strrchr($target_file_name, '.'), 1);
									$target_file_name = self::GetRandomString(20).".".$ext;
								}else{
									$target_file_name = basename($_FILES[$v_key]['name']);
								}
								$target = $target.$target_file_name; 
								$ok=1; 
								if(move_uploaded_file($_FILES[$v_key]['tmp_name'], $target)){
									$arrUploadedFiles[$v_key] = $target_file_name;
									//echo "The file ". basename( $_FILES[$key]['name']). " has been uploaded";
								}else{
									//echo "Sorry, there was a problem uploading your file.";
								}
							}
						}
					}
				}
			}else{
				if(isset($val['type']) && ($val['type'] == "image" || $val['type'] == "file")){
					if(isset($_FILES[$key]['name'])){
						if($system == "windows"){
							$target = str_replace("index.php", "", $_SERVER['SCRIPT_FILENAME']).$val['target'];
							$target = str_replace("/", "\\", $target);
							$target = str_replace("\\", "\\\\", $target);
						}else{
							$target = $val['target'];
						}								
						
						$random_name = isset($val['random_name']) ? $val['random_name'] : false; 
						if($random_name == "true" || $random_name === true){									
							$target_file_name = basename($_FILES[$key]['name']);
							$ext = substr(strrchr($target_file_name, '.'), 1);
							$target_file_name = self::GetRandomString(20).".".$ext;
						}else{
							$target_file_name = basename($_FILES[$key]['name']);
						}
						$target = $target.$target_file_name; 
						$ok=1; 
						if(move_uploaded_file($_FILES[$key]['tmp_name'], $target)){
							$arrUploadedFiles[$key] = $target_file_name;
							//echo "The file ". basename( $_FILES[$key]['name']). " has been uploaded";
						}else{
							//echo "Sorry, there was a problem uploading your file.";
						}
					}
				}
			}
		}			
		
		return $arrUploadedFiles;
	}

	/**
	 *	Remove (delete) image/file and update table field
	*/	
	private function RemoveFileImage($rid, $operation_field, $target_path, $target_file){
		//----------------------------------------------------------------------
		// block if this is a demo mode
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;				
		}

		@unlink($target_path.$target_file);
		$sql = "UPDATE ".$this->tableName." SET ".$operation_field." = '' WHERE ".$this->primaryKey." = ".$rid;
		database_void_query($sql);
		if($this->debug) $this->arrSQLs['delete_image'] = $sql;					
	}
	
	/**
	 * Prepare images/files fields
	 **/
	protected function PrepareImagesArray($rid = "0"){
		$this->arrImagesFields = array();

		// prepare images/files fields
		foreach($this->arrEditModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info"){						
						// prepare images
						if($v_val['type'] == "image" || $v_val['type'] == "file"){
							$sql = "SELECT ".$v_key." FROM ".$this->tableName." WHERE ".$this->primaryKey." = ".(int)$rid."";
							$result = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);
							if(isset($result[$v_key])){
								$this->arrImagesFields[$v_key] = $v_val['target'].$result[$v_key];	
							}							
						}											
					}					
				}				
			}else{
				// prepare images
				if($val['type'] == "image" || $val['type'] == "file"){
					$sql = "SELECT ".$key." FROM ".$this->tableName." WHERE ".$this->primaryKey." = ".(int)$rid."";
					$result = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);
					if(isset($result[$key])){
						$this->arrImagesFields[$key] = $val['target'].$result[$key];	
					}							
				}
			}
		}		
	}
	
	/**
	 * Delete images from images array
	 **/
	protected function DeleteImages(){		
		//----------------------------------------------------------------------
		// block if this is a demo mode
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;				
		}

		foreach($this->arrImagesFields as $key => $val){
			@unlink($val);
		}			
	}
	
	/**
	 *	Get formatted microtime
	*/	
    protected function GetFormattedMicrotime(){
        list($usec, $sec) = explode(' ', microtime());
        return ((float)$usec + (float)$sec);
    }    
	
	/**
	 *	Set running time
	*/	
    protected function SetRunningTime(){		
        if($this->debug){
            $this->startTime = $this->GetFormattedMicrotime();
        }        
    }    
		
	/**
	 *	Draw script running time
	*/	
    protected function DrawRunningTime(){
        // finish calculating running time of a script
        if($this->debug){
            $this->finishTime = $this->GetFormattedMicrotime();
            $output = "<br><div style='width:100%; text-align:left; color:#000047;'>";
			$output .= "Total running time: ".round((float)$this->finishTime - (float)$this->startTime, 6)." sec.";
			$output .= "</div>";
			echo $output;
        }        
    }    

	////////////////////////////////////////////////////////////////////////////
	static public function GetParameter($param, $use_prefix = true, $u_prefix = ""){
		$prefix = ($use_prefix) ? "mg_" : "";
		$output = isset($_REQUEST[$u_prefix.$prefix.$param]) ? $_REQUEST[$u_prefix.$prefix.$param] : "";
		return $output;
	}
		
    static public function GetRandomString($length = 20) {
        $template_alpha = "abcdefghijklmnopqrstuvwxyz";
        $template_alphanumeric = "1234567890abcdefghijklmnopqrstuvwxyz";
        settype($template, "string");
        settype($length, "integer");
        settype($rndstring, "string");
        settype($a, "integer");
        settype($b, "integer");
        $b = rand(0, strlen($template_alpha) - 1);
        $rndstring .= $template_alpha[$b];        
        for ($a = 0; $a < $length-1; $a++) {
            $b = rand(0, strlen($template_alphanumeric) - 1);
            $rndstring .= $template_alphanumeric[$b];
        }       
        return $rndstring;       
    }
		
}
?>