<?php

/**
 *	Class ModulesSettings
 *  -------------- 
 *  Description : encapsulates Modules Settings operations & properties
 *  Updated	    : 28.06.2010
 *  Version		: 1.0.2
 *	Written by  : ApPHP
 *
 *	__construct
 *	__destruct
 *
 *	PUBLIC
 *	------------------
 *	UpdateRecord
 *	GetSettings
 *
 *	DONE:
 *	1.0.2 :
 *		- added "text" field type - WHEN key_display_type = 'text' THEN '' in viewmode
 *		- added required/non-required fields ALTER TABLE `mb_modules_settings` ADD `key_required` TINYINT( 1 ) UNSIGNED NOT NULL DEFAULT '1' AFTER `key_display_type` 
 *	
 **/

class ModulesSettings extends MicroGrid {
	
	protected $debug = false;

	protected $tableName;
	protected $primaryKey;
	protected $dataSet;
	protected $formActionURL;
	protected $params;
	protected $actions;
	protected $isHtmlEncoding;

	protected $languageId;
	protected $allowLanguages;

	protected $VIEW_MODE_SQL;
	protected $EDIT_MODE_SQL;
	protected $DETAILS_MODE_SQL;
	protected $WHERE_CLAUSE;
	protected $ORDER_CLAUSE;
	protected $LIMIT_CLAUSE;

	protected $arrViewModeFields;	
	protected $arrAddModeFields;	
	protected $arrEditModeFields;
	protected $arrDetailsModeFields;
	
	protected $isAlterColorsAllowed;
	
	protected $isPagingAllowed;
	protected $pageSize;
	
	protected $isSortingAllowed;
	
	private $moduleName;
	private $arrSettings;

	public $error;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct($module_name = "") {
		
		$this->params = array();		
		if(isset($_POST['settings_description'])) $this->params['settings_description'] = $_POST['settings_description'];
		if(isset($_POST['settings_key']))   	  $this->params['settings_key']   = $_POST['settings_key'];
		if(isset($_POST['settings_value']))       $this->params['settings_value'] = $_POST['settings_value'];
		if(isset($_POST['module_name']))          $this->params['module_name']    = $_POST['module_name'];
		if(isset($_POST['key_display_type']))     $this->params['key_display_type'] = $_POST['key_display_type'];

		$this->moduleName   = $module_name;
		$this->primaryKey 	= "id";
		$this->tableName 	= DB_PREFIX."modules_settings";
		$this->dataSet 		= array();
		$this->error 		= "";
		$this->formActionURL = "index.php?admin=mod_".$this->moduleName."_settings";
		$this->actions      = array("add"=>false, "edit"=>true, "details"=>true, "delete"=>false);

		$this->isAlterColorsAllowed = true;

		$this->isPagingAllowed = true;
		$this->pageSize = 10;

		$this->isSortingAllowed = true;

		// save module settings
		$this->arrSettings = array();
		$sql = "SELECT settings_key, settings_value, key_is_required
		        FROM ".$this->tableName."
				WHERE module_name = '".$this->moduleName."'";
		$result = database_query($sql, DATA_AND_ROWS, ALL_ROWS);
		for($i=0; $i < $result[1]; $i++){
			$this->arrSettings[$result[0][$i]['settings_key']] = array("value"=>$result[0][$i]['settings_value'], "required"=>$result[0][$i]['key_is_required']);	
		}

		//---------------------------------------------------------------------- 
		// VIEW MODE
		//---------------------------------------------------------------------- 
		$this->VIEW_MODE_SQL = "SELECT ".$this->primaryKey.",
									settings_key,
									CASE
										WHEN key_display_type = 'enum' OR key_display_type = 'yes/no' THEN CONCAT(UCASE(MID(settings_value,1,1)),MID(settings_value,2))
										WHEN key_display_type = 'text' THEN ''
										ELSE settings_value
									END as settings_value,
									settings_description,
									key_display_type
								FROM ".$this->tableName."
								WHERE module_name = '".$this->moduleName."'";		
		$this->arrViewModeFields = array(
			"settings_description" => array("title"=>_NAME,  "type"=>"label", "width"=>"70%", "align"=>"left"),
			"settings_value"       => array("title"=>_VALUE, "type"=>"label", "width"=>"", "align"=>"center")
		);
		
		//---------------------------------------------------------------------- 
		// ADD MODE
		//---------------------------------------------------------------------- 
		// define add mode fields
		$arr_key_display_types = array("string"=>"string", "numeric"=>"numeric", "enum"=>"enum");
		$this->arrAddModeFields = array(
			"settings_description" 	=> array("title"=>_NAME,  "type"=>"textbox", "required"=>true),		
			"settings_key"   		=> array("title"=>_KEY,   "type"=>"textbox", "required"=>true),
			"settings_value" 		=> array("title"=>_VALUE, "type"=>"textbox", "required"=>true),			
			"key_display_type"      => array("title"=>_KEY_DISPLAY_TYPE, "type"=>"enum", "source"=>$arr_key_display_types, "required"=>true),
			"module_name"    		=> array("title"=>"",    "type"=>"hidden",  "required"=>true, "default"=>$this->moduleName),
		);

		//---------------------------------------------------------------------- 
		// EDIT MODE
		//---------------------------------------------------------------------- 
		$this->EDIT_MODE_SQL = "SELECT ".$this->primaryKey.",
								settings_key,
								settings_value,
								settings_description,
								key_display_type
							FROM ".$this->tableName."
							WHERE ".$this->primaryKey." = _RID_";		
		// define edit mode fields
		$this->arrEditModeFields = array(
			"settings_description" 	=> array("title"=>_NAME,  "type"=>"label"),		
			"settings_value" 		=> array("title"=>_VALUE, "type"=>"textbox"),
			"settings_key"   		=> array("title"=>"",   "type"=>"hidden"),
			"key_display_type"   	=> array("title"=>"",   "type"=>"hidden"),			
		);

		//---------------------------------------------------------------------- 
		// DETAILS MODE
		//----------------------------------------------------------------------
		$this->DETAILS_MODE_SQL = $this->EDIT_MODE_SQL;
		$this->arrDetailsModeFields = array(
			"settings_description" 	=> array("title"=>_NAME,  "type"=>"label"),
			"settings_value" 		=> array("title"=>_VALUE, "type"=>"label"),
			"settings_key"   		=> array("title"=>_KEY,   "type"=>"hidden"),			
		);

	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct()
	{
		// echo 'this object has been destroyed';
    }


	/***
	 *	Get all DataSet array
	 *
	 *  public static function GetAll() { }
	 *	
	 **/
	
	/***
	 *	Add record
	 *
	 *	function function AddRecord($params = array()) { 	}
	 *	
	 **/

	/***********************************************************************
	 *
	 *	Draw Edit Mode
	 *	
	 ***********************************************************************/
	function DrawEditMode($rid = "0", $operation = "", $operation_field = ""){
		
		$this->IncludeJSFunctions();
		
		$sorting_fields  = self::GetParameter('sorting_fields');
		$sorting_types   = self::GetParameter('sorting_types');
		$page 			 = self::GetParameter('page');
		$operation 		 = self::GetParameter('operation');
		$operation_type  = self::GetParameter('operation_type');
		$operation_field = self::GetParameter('operation_field');
		$search_status   = self::GetParameter('search_status');
		$is_required 	 = false;
		
		echo "\n<form name='frmMicroGrid_".$this->tableName."' id='frmMicroGrid_".$this->tableName."' action='".$this->formActionURL."' method='post' enctype='multipart/form-data'>\n";
		echo draw_hidden_field("mg_action", "update")."\n";
		echo draw_hidden_field("mg_rid", $rid)."\n";
		echo draw_hidden_field("mg_sorting_fields", $sorting_fields)."\n";
		echo draw_hidden_field("mg_sorting_types", $sorting_types)."\n";
		echo draw_hidden_field("mg_page", $page)."\n";
		echo draw_hidden_field("mg_operation", "")."\n";
		echo draw_hidden_field("mg_operation_type", "")."\n";
		echo draw_hidden_field("mg_operation_field", "")."\n";
		echo draw_hidden_field("mg_search_status", $search_status)."\n";
		echo draw_hidden_field("mg_language_id", $this->languageId)."\n";
		// save filter (search) data for view mode
		if($this->isFilteringAllowed){
			foreach($this->arrFilteringFields as $key => $val){
				if($val['type'] == "text"){
					$filter_field_value = ($search_status == "active") ? $filter_field_value = self::GetParameter("filter_by_".$key, false) : "";
					echo draw_hidden_field("filter_by_".$key, $filter_field_value)."\n";
				}
			}
		}

		// prepare password fields
		foreach($this->arrEditModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info"){						
						if($v_val['type'] == "password"){
							$password_field = $this->UncryptPasswordValue($v_key, $v_val);
							str_replace($v_key, $password_field, $this->arrEditModeFields);
						}											
					}					
				}				
			}else{
				if($val['type'] == "password"){
					$password_field = $this->UncryptPasswordValue($key, $val);
					$this->EDIT_MODE_SQL = str_replace($this->tableName.".".$key, $password_field, $this->EDIT_MODE_SQL);
				}									
			}
		}				

		$this->EDIT_MODE_SQL = str_replace("_RID_", $rid, $this->EDIT_MODE_SQL);
		$result = database_query($this->EDIT_MODE_SQL, DATA_AND_ROWS);		
		if($this->debug) $this->arrSQLs['select_edit_mode'] = $this->EDIT_MODE_SQL;					

		// draw hidden fields
		foreach($this->arrEditModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info"){						
						// delete file/image
						if($operation == "remove" && $operation_field != "" && ($v_key == $operation_field)){
							$this->RemoveFileImage($rid, $operation_field, $v_val['target'], $result[0][0][$v_key]);
							$result[0][0][$v_key] = "";
						}
						if($v_val['type'] == "hidden"){
							draw_hidden_field($v_key, $result[0][0][$v_key]);
							echo "\n";
						}
					}					
				}				
			}else{
				// delete file/image
				if($operation == "remove" && $operation_field != "" && ($key == $operation_field)){
					$this->RemoveFileImage($rid, $operation_field, $val['target'], $result[0][0][$key]);
					$result[0][0][$key] = "";
				}
				if($val['type'] == "hidden"){
					if(isset($this->arrSettings[$result[0][0][$key]]['required']) && $this->arrSettings[$result[0][0][$key]]['required'] == "1") $is_required = true;
					draw_hidden_field($key, $result[0][0][$key]);
					echo "\n";
				}
			}
		}			
		

		// draw Edit Form
		echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>";
		foreach($this->arrEditModeFields as $key => $val){
			if(preg_match("/separator/i", $key) && is_array($val)){
				echo "</table><br />\n";
				echo "<fieldset style='padding:5px;'>";
				echo "<legend>".$val['separator_info']['legend']."</legend>";
				echo "\n<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";		
				foreach($val as $v_key => $v_val){
					if($v_key != "separator_info"){						
						echo "<tr>\n";
						echo "  <td width='27%'>".$v_val['title'].(($is_required) ? " <font color='#c13a3a'>*</font>" : "").":</td>\n";
						if(isset($this->params[$v_key]) && $this->params[$v_key] !== "" && ($v_val['type'] != "checkbox")){
							echo "  <td style='padding-left:6px;'>".$this->DrawFieldByType("edit", $v_key, $v_val, $this->params, false)."</td>\n";
						}else{
							echo "  <td style='padding-left:6px;'>".$this->DrawFieldByType("edit", $v_key, $v_val ,$result[0][0], false)."</td>\n";
						}
						echo "</tr>\n";
					}
				}
				echo "</table>\n";
				echo "</fieldset>\n";				
				echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='mgrid_table'>\n";		
			}else{
				if($val['type'] != "hidden"){
					echo "<tr>\n";
					echo "  <td width='20%'>".$val['title'].(($is_required && $key == "settings_value") ? " <font color='#c13a3a'>*</font>" : "").":</td>\n";
                    
					// prepare some settings depended on field type
					if($key == "settings_value"){
						$sql = "SELECT key_display_type, key_display_source FROM ".$this->tableName." WHERE ".$this->primaryKey."=".$rid;
						if($row = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY)){								
							if($row['key_display_type'] == "enum"){
								$key_display_source = explode(",", $row['key_display_source']);
								$enum_source = array();
								foreach($key_display_source as $kds_key){
									$enum_source[$kds_key] = ucfirst($kds_key);
								}
								$val['type'] = "enum";
								$val['source'] = $enum_source;
							}else if($row['key_display_type'] == "yes/no"){
								$enum_source = array("no"=>_NO, "yes"=>_YES);
								$val['type'] = "enum";
								$val['source'] = $enum_source;									
							}else if($row['key_display_type'] == "numeric"){
								$val['width'] = "50px";
							}else if($row['key_display_type'] == "text"){
								$val['type'] = "textarea";
								$val['width'] = "530px";
								$val['height'] = "115px";
							}else if($row['key_display_type'] == "positive integer"){									
								$val['width'] = "50px";
							}else if($row['key_display_type'] == "unsigned float"){
								$val['width'] = "50px";								
							}else{
								$val['width'] = "210px";
							}
						}							
					}
					if(isset($this->params[$key]) && $this->params[$key] !== "" && ($val['type'] != "checkbox")){
						echo "  <td>".$this->DrawFieldByType("edit", $key, $val, $this->params, false)."</td>\n";													
					}else{
						echo "  <td>".$this->DrawFieldByType("edit", $key, $val, $result[0][0], false)."</td>\n";						
					}
					echo "</tr>\n";				
				}
			}
		}
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>";
		echo "<tr>
				<td colspan='2'>
					<input class='mgrid_button' type='submit' name='subUpdateRecord' value='"._BUTTON_UPDATE."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"update\");'>
					<input class='mgrid_button' type='button' name='btnCancel' value='"._BUTTON_CANCEL."' onclick='javascript:__mgDoPostBack(\"".$this->tableName."\", \"view\");'>
				</td>
			  <tr>\n";		
		echo "</table><br />\n";
		echo "</form>\n";

		if($this->errorField != "") echo "<script type='text/javascript'>__mgSetFocus('".$this->errorField."');</script>";	   
		
		$this->DrawRunningTime();
		$this->DrawErrors();
		$this->DrawWarnings();
		$this->DrawSQLs();	
		$this->DrawPostInfo();	
	}



	/***************************************************************************
	 *
	 *	UPDATE RECORD
	 *	
	 **************************************************************************/
	function UpdateRecord($rid = "0") {

		//----------------------------------------------------------------------
		// check if all required fields not empty
		if($rid == "0" || $rid == "" || !is_numeric($rid)){
			$this->error = _WRONG_PARAMETER_PASSED;
			return false;
		}
		
		foreach($this->arrEditModeFields as $key => $val){
			
			if($key == "settings_key"){
				
				$sql = "SELECT key_display_type, key_is_required, settings_value 
						FROM ".$this->tableName."
					    WHERE settings_key = '".$this->params[$key]."' AND module_name = '".$this->moduleName."'";					
				if($row = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY)){
					if(($row['key_is_required'] == "1") && ($this->params['settings_value'] == "")){
						$this->error = str_replace("_FIELD_", $val['title'], _FIELD_CANNOT_BE_EMPTY);
						return false;					
					}
					
					if($row['key_display_type'] == "email" && !check_email_address($this->params['settings_value'])){
						$this->error = str_replace("_FIELD_", "PayPal Email", _FIELD_MUST_BE_EMAIL);
						return false;						
					}else if($row['key_display_type'] == "numeric" && !is_numeric($this->params['settings_value'])){
						$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_NUMERIC);
						return false;
					}else if($row['key_display_type'] == "positive integer" && (!$this->IsInteger($this->params['settings_value']) || ($this->params['settings_value'] <= 0))){
						$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_POSITIVE_INT);
						return false;						
					}else if($row['key_display_type'] == "unsigned float" && (!$this->IsFloat($this->params['settings_value']) || ($this->params['settings_value'] < 0))){
						$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_UNSIGNED_FLOAT);
						return false;						
					}else if($row['key_display_type'] == "yes/no" && (strtolower($this->params['settings_value']) != "yes" && strtolower($this->params['settings_value']) != "no")){
						$this->error = str_replace("_FIELD_", $val['title'], _FIELD_MUST_BE_BOOLEAN);
						return false;						
					}
				}					
			}			
		}
		 
		//----------------------------------------------------------------------
		// block if this is a demo mode
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;				
		}
		
		//----------------------------------------------------------------------
		// update
		$sql = "UPDATE ".$this->tableName." SET ";
			$fields_count = 0;
			foreach($this->params as $key => $val){
				if(array_key_exists($key, $this->arrEditModeFields)){
					if($fields_count++ > 0) $sql .= ",";
					$sql .= $key." = '".$val."'";					
				}
			}				
		$sql .= " WHERE ".$this->primaryKey."=".(int)$rid;
		
		if(database_void_query($sql, $this->debug) < 0){
			$this->error = _TRY_LATER;
			return false;
		}else{
			return true;
		}	
		
	}
	
	/***
	 *	Get module parameter
	 *
	 *		@param $param_name
	 *	
	 **/
	public function GetSettings($param_name = "")
	{
		return isset($this->arrSettings[$param_name]['value']) ? $this->arrSettings[$param_name]['value'] : "";
	}

	/***
	 *	Delete record
	 *
	 *	function DeleteRecord($rid = "") { 	}
	 *	
	 **/
	


}
?>