<?php

/***
 *	Class Posts
 *  -------------- 
 *  Description : encapsulates posts properties
 *  Updated	    : 25.04.2010
 *	Written by  : ApPHP
 *	
 **/

class Posts {

	protected $post_key;
	protected $post_id;
	protected $post;
	
	private $postSize;
	private $totalSearchRecords;

	public $error;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct($post_id = "", $is_active = false) {
		
		$this->postSize = 20;
		$this->totalSearchRecords = 0;		
	
		if($post_id != ""){
			$sql = "SELECT
						".DB_PREFIX."posts.*
					FROM ".DB_PREFIX."posts
					WHERE
						".DB_PREFIX."posts.id = '".(int)$post_id."'";
					if($is_active){
						$sql .= " AND ".DB_PREFIX."posts.is_published = 1 ";
					}
			$this->post = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);
			if(empty($this->post)){
				$this->post_id = "";
			}else{
				$this->post_id = $post_id;				
			}			
		}else{
			$this->post_id = $post_id;
			$this->post['id'] = "";
			$this->post['author_id'] = "0";
			$this->post['post_title'] = "";
			$this->post['post_key'] = "";
			$this->post['post_text'] = "";
			$this->post['category_id'] = "0";
			$this->post['tags'] = "";
			$this->post['comments_allowed'] = "0";
			$this->post['is_published'] = "0";
			$this->post['is_rss_published'] = "1";
		}
	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct(){
		// echo 'this object has been destroyed';
    }

	/***
	 * Draw post title
	 *		@param $template - template name
	 **/
	public function DrawTitle($template = "orange"){
		if(isset($this->post['post_title'])){
			$post_title = decode_text($this->post['post_title']);
			draw_title_bar($post_title);
		}
	}

	/***
	 *	Get parameter
	 *	
	 **/
	public function GetParameter($param = ""){
		if(isset($this->post[$param])){
			return $this->post[$param];
		}else{
			return "";
		}
	}

	/***
	 * Return post title
	 *		
	 **/
	public function GetTitle(){
		if(isset($this->post['post_title'])){
			$post_title = decode_text($this->post['post_title']);
			return $post_title;
		}else return "";
	}	

	/***
	 * Return menu ID
	 *		
	 **/
	public function GetCategoryId(){
		if(isset($this->post['category_id'])) return $this->post['category_id'];
		return "";
	}


	/***
	 * Return post key
	 *		
	 **/
	public function GetKey(){
		return $this->post_key;
	}

	/***
	 * Return post ID
	 *		
	 **/
	public function GetId(){
		return $this->post_id;
	}
	
	/***
	 * Return post text
	 *		
	 **/
	public function GetText(){
		if(isset($this->post['post_text'])){
			$post_text = decode_text($this->post['post_text'], false);
			return $post_text;
		}else return "";
	}
	
	/***
	 * Return record set of current post
	 *		
	 **/
	function GetAll(){		
		$where_clause = "1=1 ";
		$order_clause = "";

		$sql = "SELECT
					".DB_PREFIX."posts.*
				FROM ".DB_PREFIX."posts  
					LEFT OUTER JOIN ".DB_PREFIX."categories ON ".DB_PREFIX."posts.category_id=".DB_PREFIX."categories.id
				WHERE 
					".$where_clause."
					".$order_clause."";				
		return database_query($sql, DATA_AND_ROWS);
	}
	
	/***
	 * Return record set of all active posts
	 *		
	 **/
	function GetAllActive(){
		$sql = "SELECT
					".DB_PREFIX."posts.*
				FROM ".DB_PREFIX."posts  
					LEFT OUTER JOIN ".DB_PREFIX."categories ON ".DB_PREFIX."posts.category_id=".DB_PREFIX."categories.id
				WHERE ".DB_PREFIX."posts.is_published = 1
				ORDER BY ".DB_PREFIX."posts.posted_at DESC
				LIMIT 0, 10";				
		return database_query($sql, DATA_AND_ROWS);		
	}

	/***
	 * Search in posts by keyword
	 *		@param $keyword - keyword
	 **/	
	function SearchBy($keyword, $post = 1){
		
		$keyword = encode_text($keyword);
		
		$sql = "SELECT
					id,
					post_title,
					post_text 
				FROM ".DB_PREFIX."posts  
				WHERE
					is_published = 1 AND
					( post_title LIKE '%".$keyword."%' OR
					post_text LIKE '%".$keyword."%' ) ";

		if(!is_numeric($post) || (int)$post <= 0) $post = 1;
		$this->totalSearchRecords = (int)database_query($sql, ROWS_ONLY);
		$total_posts = (int)($this->totalSearchRecords / $this->postSize);			
		if(($this->totalSearchRecords % $this->postSize) != 0) $total_posts++;
		$start_row = ($post - 1) * $this->postSize;
		
		return database_query($sql." ORDER BY id ASC LIMIT ".$start_row.", ".$this->postSize, DATA_AND_ROWS);		
	}
	
	/***
	 * Draw search result
	 *		@param $search_result - search result
	 **/	
	function DrawSearchResult($search_result, $post = 1){		
		$total_posts = (int)($this->totalSearchRecords / $this->postSize);
		if(!is_numeric($total_posts) || (int)$total_posts <= 0) $total_posts = 1;

		if($search_result != "" && $search_result[1] > 0){
			echo "<div class='search_result'>";		
			for($i = 0; $i < $search_result[1]; $i++){		
				echo ($i+1).". <a href='index.php?page=posts&post_id=".$search_result[0][$i]['id']."'>".$search_result[0][$i]['post_title']."</a><br />";
				
				$post_text = $search_result[0][$i]['post_text'];
				$post_text = preg_replace("/{module:gallery}/i", "", $post_text);
				$post_text = preg_replace("/{module:contact}/i", "", $post_text);
				
				echo substr_by_word(strip_tags(decode_text($post_text)), 255)."...<br /><br />";
				draw_line();		
				echo "<br />";
			}
			echo "<b>"._POSTS.":</b> ";
			for($i = 1; $i <= $total_posts; $i++){
				echo "<a class='pagging_link' href='javascript:void(0);' onclick='javascript:appPerformSearch(".$i.");'>".(($i == $post) ? "<b>[".$i."]</b>" : $i)."</a> ";
			}
			echo "</div>";
		}else{
			draw_important_message(_NO_RECORDS_FOUND);		
		}		
		
	}
	
	/***
	 * Update current post
	 *		@param $params - set of fields
	 **/
	function PostUpdate($params = array()) {
		if(isset($this->post['id'])){
			if(strtolower(SITE_MODE) == "demo"){
				$this->error = _OPERATION_BLOCKED;
				return false;
			}else{				
			
				// Get input parameters
				if(isset($params['author_id']))	     $this->post['author_id'] = $params['author_id'];
				if(isset($params['post_title']))	 $this->post['post_title'] = $params['post_title'];
				if(isset($params['post_key'])) 		 $this->post['post_key'] = $params['post_key'];
				if(isset($params['post_text']))		 $this->post['post_text'] = $params['post_text'];
				if(isset($params['category_id']))	 $this->post['category_id'] = $params['category_id'];
				if(isset($params['is_published']))	 $this->post['is_published'] = $params['is_published'];
				if(isset($params['is_rss_published'])) $this->post['is_rss_published'] = $params['is_rss_published'];
				if(isset($params['comments_allowed'])) $this->post['comments_allowed'] = $params['comments_allowed'];
				if(isset($params['tags'])) 		     $this->post['tags'] = $params['tags'];
				if(isset($params['posted_at']))      $this->post['posted_at'] = $params['posted_at'];

				if(isset($params['tag_title']))   	 $this->post['tag_title'] = $params['tag_title'];
				if(isset($params['tag_keywords']))   $this->post['tag_keywords'] = $params['tag_keywords'];
				if(isset($params['tag_description']))$this->post['tag_description'] = $params['tag_description'];		
				
				// Menu link cannot be more then 30 characters
				if($this->post['post_title'] == ""){
					$this->error = _POST_HEADER_EMPTY;
					return false;			
				}
				
				$post_text = $this->post['post_text']; 
				$post_title = encode_text($this->post['post_title']);
				$tags 	= encode_text($this->CleanTags($this->post['tags']));
				
				$sql = "SELECT tags FROM ".DB_PREFIX."posts WHERE id = '".(int)$this->post['id']."'";
				$result = database_query($sql, DATA_AND_ROWS, FIRST_ROW_ONLY);
				if($result[1] > 0) $old_tags = $result[0]['tags'];
				else $old_tags = "";
				
				$sql = "UPDATE ".DB_PREFIX."posts
						SET
							post_title 	 = '".encode_text($post_title)."',
							post_key 	 = '".$this->post['post_key']."',
							post_text 	 = '".encode_text($post_text)."',
							category_id  = ".(int)$this->post['category_id'].",
							tag_title    = '".encode_text($this->post['tag_title'])."',
							tag_keywords = '".encode_text($this->post['tag_keywords'])."',
							tag_description = '".encode_text($this->post['tag_description'])."',
							comments_allowed = ".(int)$this->post['comments_allowed'].",
							tags = '".$tags."',
							is_published = ".(int)$this->post['is_published'].",
							is_rss_published = ".(int)$this->post['is_rss_published']."
							".(($this->post['posted_at'] != "") ? ", posted_at = '".$this->post['posted_at']."'" : "")."
						WHERE id = '".(int)$this->post['id']."'";						
				if(database_void_query($sql)){
					if($this->post['is_published']) Tags::UpdateTags(explode(",", $old_tags), explode(",", $tags));
					return true;
				}else{
					$this->error = _TRY_LATER;
					return false;
				}				
			}
		}else{
			$this->error = _PAGE_UNKNOWN;
			return false;
		}
	}
	
	/***
	 * Create new post
	 *		@param $params - set of fields
	 **/
	function PostCreate($params = array()) {
		// Get input parameters
		if(isset($params['author_id']))		$this->post['author_id'] = $params['author_id'];
		if(isset($params['post_title']))	$this->post['post_title'] = $params['post_title'];
		if(isset($params['post_key']))		$this->post['post_key'] = $params['post_key'];
		if(isset($params['post_text']))		$this->post['post_text'] = $params['post_text'];
		if(isset($params['category_id']))	$this->post['category_id'] = $params['category_id'];
		if(isset($params['is_published']))  $this->post['is_published'] = $params['is_published'];
		if(isset($params['is_rss_published'])) $this->post['is_rss_published'] = $params['is_rss_published'];
		if(isset($params['comments_allowed'])) $this->post['comments_allowed'] = $params['comments_allowed'];
		if(isset($params['tags'])) 	    $this->post['tags'] = $params['tags'];

		if(isset($params['tag_title']))   	 $this->post['tag_title'] = $params['tag_title'];
		if(isset($params['tag_keywords']))   $this->post['tag_keywords'] = $params['tag_keywords'];
		if(isset($params['tag_description']))$this->post['tag_description'] = $params['tag_description'];		

		// Menu link cannot be more then 30 characters
		if($this->post['post_title'] == ""){
			$this->error = _POST_HEADER_EMPTY;
			return false;			
		}
		
		$post_text 	= $this->post['post_text']; 
		$post_title = encode_text($this->post['post_title']);
		$tags 	= encode_text($this->CleanTags($this->post['tags']));
	
		if(strtolower(SITE_MODE) == "demo"){
			$this->error = _OPERATION_BLOCKED;
			return false;
		}else{				
			// Create new record
			$sql = "INSERT INTO ".DB_PREFIX."posts(
					id,
					author_id,
					post_key,
					post_title,
					post_text,
					category_id,
					tag_title,
					tag_keywords,
					tag_description,
					comments_allowed,
					tags,
					is_published,
					is_rss_published,
					posted_at,
					status_changed
				)VALUES(
					NULL,
					".(int)$this->post['author_id'].",
					'',
					'".encode_text($post_title)."',
					'".encode_text($post_text)."',
					".(int)$this->post['category_id'].",
					'".encode_text($this->post['tag_title'])."',
					'".encode_text($this->post['tag_keywords'])."',
					'".encode_text($this->post['tag_description'])."',
					".(int)$this->post['comments_allowed'].",
					'".$tags."',
					".(int)$this->post['is_published'].",
					".(int)$this->post['is_rss_published'].",
					'".date("Y-m-d H:i:s")."',
					'".date("Y-m-d H:i:s")."'
				)";
				//".(int)$this->GetMenuIdBy($this->post['category_id']).",
				
			if(database_void_query($sql)){
				// Update post_key
				$last_insert_id = database_insert_id();
				$sql = "UPDATE ".DB_PREFIX."posts 
						SET post_key='".$this->post['post_key']."'
						WHERE id=".intval($last_insert_id)."";
				if(database_void_query($sql)){
					// ok
					if($this->post['is_published']) Tags::AddTags(explode(",", $tags));
				}else{
					$this->error = _TRY_LATER;
					return false;
				}
			}else{
				$this->error = _TRY_LATER;
				return false;
			}					

			return true;				
		}			
		
	}

	/***
	 * Delete post
	 *		@param $post_id
	 **/
	function PostDelete($post_id = "") {
		// check that the post exists		
		if($this->post_id != ""){
			// Delete post
			if(strtolower(SITE_MODE) == "demo"){
				$this->error = _OPERATION_BLOCKED;
				return false;
			}else{
				$sql = "SELECT tags FROM ".DB_PREFIX."posts WHERE id = '".(int)$post_id."'";
				$result = database_query($sql, DATA_AND_ROWS, FIRST_ROW_ONLY);
				if($result[1] > 0) $tags = $result[0]['tags'];
				else $tags = "";				
				
				$sql = "DELETE FROM ".DB_PREFIX."posts WHERE id = '".(int)$post_id."'";
				if(database_void_query($sql)){
					if($tags != "") Tags::DeleteTags(explode(",", $tags));
					return true;
				}
			}
		}
		$this->error = _TRY_LATER;
		return false;
	}

	/***
	 * Get menu id by
	 *		@param $category_id
	 **/
	private function GetCategoryIdBy($category_id = "0"){
		$sql = "SELECT ".DB_PREFIX."categories.menu_code
				FROM ".DB_PREFIX."categories
				WHERE ".DB_PREFIX."categories.id = '".$category_id."'";
		$menu = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);
		if(!empty($menu)){
			$sql = "SELECT ".DB_PREFIX."menus.id
					FROM ".DB_PREFIX."menus
					WHERE
						".DB_PREFIX."menus.menu_code = '".$menu['menu_code']."' ";
			$category_id = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);
			if(!empty($category_id)){
				return $category_id['id'];
			}
		}
		return "0";
	}
	

	/***
	 * Check if user has access to edit post
	 *		@param $post_id
	 *		@param $author_id
	 **/
	function HasAccess($post_id, $author_id)
	{
		if(!empty($author_id) && !empty($post_id)){
			$sql = "SELECT id FROM ".DB_PREFIX."posts WHERE id = ".(int)$post_id." AND author_id = ".(int)$author_id;
			$result = database_query($sql, DATA_AND_ROWS);		
			if($result[1] > 0) return true;
		}
		return false;
	}

	/***
	 * Draw Posts
	 *		@param $cat_id
	 **/
	function DrawPosts($type, $cat_id, $updated_max, $reverse)
	{
		global $objSettings;
		
		$objAdsenseSettings = new ModulesSettings("adsense");
		$minimal_posts	    = $objAdsenseSettings->GetSettings("minimal_posts");
		
		$updated_min = "";
		$last_posted = "";
		if(strlen($updated_max) == 19){
			$updated_max_parts  = explode(".", $updated_max);
			$updated_max_year   = isset($updated_max_parts[0]) ? convert_to_decimal((int)$updated_max_parts[0]) : date("Y");
			$updated_max_month  = isset($updated_max_parts[1]) ? convert_to_decimal((int)$updated_max_parts[1]) : date("m");
			$updated_max_day    = isset($updated_max_parts[2]) ? convert_to_decimal((int)$updated_max_parts[2]) : date("d");
			$updated_max_hour   = isset($updated_max_parts[3]) ? convert_to_decimal((int)$updated_max_parts[3]) : date("H");
			$updated_max_min    = isset($updated_max_parts[4]) ? convert_to_decimal((int)$updated_max_parts[4]) : date("i");
			$updated_max_sec    = isset($updated_max_parts[5]) ? convert_to_decimal((int)$updated_max_parts[5]) : date("s");
			$updated_max        = $updated_max_year."-".$updated_max_month."-".$updated_max_day." ".$updated_max_hour.":".$updated_max_min.":".$updated_max_sec;
		}
		
		$output = "";
		$this->category_id = $cat_id;

		$where_clause_type = "";
		$where_clause_posted_at = "";
		$order_direction = "DESC";
		if($type == "category"){
			if($cat_id == "-1"){
				// do nothing
			}else if($cat_id == "0"){
				$where_clause_type = " AND ".DB_PREFIX."posts.category_id = 0";
			}else if($cat_id > 0){
				$where_clause_type = " AND ".DB_PREFIX."posts.category_id = ".(int)$this->category_id;				
			}
		}else{
			$where_clause_type = " AND ".DB_PREFIX."posts.tags LIKE '%".$cat_id."%'";			
		}
		if($updated_max != ""){
			if($reverse == "1"){
				$where_clause_posted_at = " AND ".DB_PREFIX."posts.posted_at > '".$updated_max."'";
				$order_direction = "ASC";
			}else{
				$where_clause_posted_at = " AND ".DB_PREFIX."posts.posted_at < '".$updated_max."'";
			}
		}
	
		$sql = "SELECT
					MAX(".DB_PREFIX."posts.posted_at) as max_posted_at,
					MIN(".DB_PREFIX."posts.posted_at) as min_posted_at
				FROM ".DB_PREFIX."posts
					LEFT OUTER JOIN ".DB_PREFIX."categories ON ".DB_PREFIX."posts.category_id = ".DB_PREFIX."categories.id
					LEFT OUTER JOIN ".DB_PREFIX."accounts  ON ".DB_PREFIX."posts.author_id=".DB_PREFIX."accounts.id
				WHERE
					".DB_PREFIX."posts.is_published = 1 
					".$where_clause_type."
				ORDER BY ".DB_PREFIX."posts.posted_at DESC
				LIMIT 0,1";
		$posts = database_query($sql, DATA_AND_ROWS, FIRST_ROW_ONLY);
		if($posts[1] > 0){
			$last_posted = $posts[0]['max_posted_at'];
			$first_posted = $posts[0]['min_posted_at'];
		}

		$sql = "SELECT
					".DB_PREFIX."posts.id as post_id,
					".DB_PREFIX."posts.category_id,
					".DB_PREFIX."posts.posted_at,
					".DB_PREFIX."posts.tags,
					".DB_PREFIX."posts.post_title,
					".DB_PREFIX."posts.post_text,
					".DB_PREFIX."posts.comments_allowed,
					IF(".DB_PREFIX."categories.category_name IS NULL, 'Other', ".DB_PREFIX."categories.category_name) as category_name,
					IF(".DB_PREFIX."accounts.user_name != '', ".DB_PREFIX."accounts.user_name, 'Admin') as author_name
				FROM ".DB_PREFIX."posts
					LEFT OUTER JOIN ".DB_PREFIX."categories ON ".DB_PREFIX."posts.category_id = ".DB_PREFIX."categories.id
					LEFT OUTER JOIN ".DB_PREFIX."accounts  ON ".DB_PREFIX."posts.author_id=".DB_PREFIX."accounts.id
				WHERE
					".DB_PREFIX."posts.is_published = 1 
					".$where_clause_type."
					".$where_clause_posted_at."
				ORDER BY ".DB_PREFIX."posts.posted_at ".$order_direction."
				LIMIT 0, ".(int)$objSettings->GetParameter("posts_on_main_page");
		$posts = database_query($sql, DATA_AND_ROWS);

		if($posts[1] > 0){
			$adsense_count = 0;
			
			if($reverse == "1"){
				$posts[0] = array_reverse($posts[0]);				
			}
			for($i=0; $i < $posts[1]; $i++){			
				
				if($i == 0) $updated_min = $posts[0][$i]['posted_at'];
				
				$post_text = $posts[0][$i]['post_text'];
				
				$post_text = preg_replace("/{module:gallery}/i", "", $post_text);
				$post_text = preg_replace("/{module:contact}/i", "", $post_text);
				
				if(strlen($post_text) > 1024){
					$post_text = substr_by_word($post_text, 1024)."...";
					$post_text .= "<br /><a href='index.php?page=posts&post_id=".$posts[0][$i]['post_id']."'>"._READ_MORE." &raquo;</a>";
				}

				$article_comments_link = "";
				$objCommentsSettings = new ModulesSettings("comments");
				$comments_allow	= $objCommentsSettings->GetSettings("comments_allow");
				if($comments_allow == "yes" && $posts[0][$i]['comments_allowed']){
					$objComments = new Comments();
					$article_comments_link = "<img src='images/comments.jpg' alt='' style='float:left;margin-right:3px;' />".$objComments->DrawArticleCommentsLink($posts[0][$i]['post_id'], "index.php?page=posts&post_id=".$posts[0][$i]['post_id']."&com_status=open");
				}
				
				// Display post row
				$output .= "<div class='post'>
				<table width='97%' align='center'>
				<tr><th align='left'><a class='post_header_a' href='index.php?page=posts&post_id=".$posts[0][$i]['post_id']."'>".$posts[0][$i]['post_title']."</a></th><tr>
				<tr>
					<td align='left' class='post_text' wrap='wrap'>".stripcslashes($post_text)."</td>
				</tr>
				<tr><td nowrap height='1px'></td></tr>
				<tr>
					<td style='color:#575757' colspan='2'>
						<table cellspacing='0' border='0' width='100%'>
						<tr>
							<td width='250px'><i>".str_replace("_AUTHOR_", $posts[0][$i]['author_name'], _POSTED_BY_AUTHOR_AT)." ".format_datetime($posts[0][$i]['posted_at'])."</i></td>
							<td align='left'>".$article_comments_link."</td>
							<td align='right'><i>"._CATEGORY_WORD.": <a href='index.php?page=posts&cat_id=".$posts[0][$i]['category_id']."'>".$posts[0][$i]['category_name']."</a></i></td>
						</tr>
						<tr>
							<td colspan='3'>								
								<i>"._TAGS.": ".$this->GetTagLinks($posts[0][$i]['tags'])."</i>
							</td>
						</tr>
						</table>
					</td>
				<tr>
				</table></div><br />";

				if(($i+1) % $minimal_posts == 0){
					$adsense_count++;
					$output .= "<div class='adsense_horizontal'>".Adsense::GetHorizontalBanerCode()."</div>";
				}
				
				$updated_max = $posts[0][$i]['posted_at'];
			}
			if($i < $minimal_posts && $adsense_count == 0){
				$output .= "<div class='adsense_horizontal'>".Adsense::GetHorizontalBanerCode()."</div>";
			}
			
			$updated_min_link = "";
			$updated_max_link = "";
			$cat_id_link = ($cat_id != "-1") ? "&cat_id=".$cat_id : "";
			if($type == "tags"){
				$cat_id_link = "&tag=".$cat_id;
			}
			if($updated_min == "" && $updated_max != ""){
				$updated_min_link = "";
				$updated_max_link = "<a href='index.php?page=posts".$cat_id_link."&updated_max=".str_replace(array("-",":"," "), ".", $updated_max)."'>"._OLDER_POSTS." &raquo;</a>";				
			}else if($updated_min != "" && $updated_max == ""){
				$updated_min_link = "<a href='index.php?page=posts".$cat_id_link."&updated_max=".str_replace(array("-",":"," "), ".", $updated_min)."&reverse=1'>&laquo; "._NEWER_POSTS."</a>";
				$updated_max_link = "";				
			}else if($updated_min > $updated_max){
				$updated_min_link = "<a href='index.php?page=posts".$cat_id_link."&updated_max=".str_replace(array("-",":"," "), ".", $updated_min)."&reverse=1'>&laquo; "._NEWER_POSTS."</a>";
				$updated_max_link = "<a href='index.php?page=posts".$cat_id_link."&updated_max=".str_replace(array("-",":"," "), ".", $updated_max)."'>"._OLDER_POSTS." &raquo;</a>";				
			}else if($updated_min == $updated_max){
				$updated_min_link = "<a href='index.php?page=posts".$cat_id_link."&updated_max=".str_replace(array("-",":"," "), ".", $updated_min)."&reverse=1'>&laquo; "._NEWER_POSTS."</a>";
				$updated_max_link = "";
			}
			if($last_posted == $updated_min) $updated_min_link = "";
			if($first_posted == $updated_max) $updated_max_link = "";

			$output .= "<table class='post_navigation_wrapper'>
				<tr>
					<td align='left'><a href='index.php'>"._HOME ."</a></td>";
					if($updated_max_link){
						$output .= "<td align='left' width='90px'>".$updated_min_link."</td>";
						$output .= "<td align='right' width='90px'>".$updated_max_link."</td>";						
					}else{
						$output .= "<td align='left' width='90px'>".$updated_min_link."</td>";
					}
			$output .= "
				</tr>
				</table>";
		}else{
			return draw_important_message(_CATEGORY_EMPTY);
		}
		return $output;
	}


	/***
	 * Draw Post for admin view
	 *		@param $post_id
	 **/
	function DrawPostAdminView($post_id){

		$objGallery = new GalleryAlbums();
		$objContact = new Contact();
		$replace_needles = 1;
		$module_post = false;
		$article_comments_link = "";
		$article_comments = "";
		$comments_status = isset($_GET['com_status']) ? $_GET['com_status'] : "";

		$output = "";
		
		$sql = "SELECT
					".DB_PREFIX."posts.id as post_id,
					".DB_PREFIX."posts.category_id,
					".DB_PREFIX."posts.posted_at,
					".DB_PREFIX."posts.post_title,
					".DB_PREFIX."posts.post_text,
					".DB_PREFIX."posts.tags,
					".DB_PREFIX."posts.comments_allowed,
					".DB_PREFIX."categories.category_name,
					IF(".DB_PREFIX."accounts.user_name != '', ".DB_PREFIX."accounts.user_name, 'Admin') as author_name
				FROM ".DB_PREFIX."posts
					LEFT OUTER JOIN ".DB_PREFIX."categories ON ".DB_PREFIX."posts.category_id=".DB_PREFIX."categories.id
					LEFT OUTER JOIN ".DB_PREFIX."accounts  ON ".DB_PREFIX."posts.author_id=".DB_PREFIX."accounts.id
				WHERE
					".DB_PREFIX."posts.id = '".(int)$post_id."'";
		$posts = database_query($sql, DATA_AND_ROWS, FIRST_ROW_ONLY);
		
		if($posts[1] > 0){
			$post_text = "";			
			if(isset($posts[0]['post_text'])){
				$post_text .= decode_text($posts[0]['post_text'], false);
				
				// include lytebox files
				if(preg_match("/{module:gallery}/i", $post_text)){
					echo "<link rel='stylesheet' href='modules/lytebox/css/lytebox.css' type='text/css' media='screen' />\n";
					echo "<script type='text/javascript' language='javascript' src='modules/lytebox/js/lytebox.js'></script>\n";
					$module_post = true;
					$post_text = preg_replace("/{module:gallery}/i", $objGallery->DrawGallery(), $post_text, 1);
				}
				if(preg_match("/{module:contact}/i", $post_text)){
					$module_post = true;
					$post_text = preg_replace("/{module:contact}/i", $objContact->DrawContactForm(), $post_text, 1);			
				}
				// draw comments form
				if(!$module_post){
					if(Modules::IsModuleInstalled("comments")){
						$objCommentsSettings = new ModulesSettings("comments");
						$comments_allow	= $objCommentsSettings->GetSettings("comments_allow");
						if($comments_allow == "yes" && $posts[0]['comments_allowed']){
							$objComments = new Comments();
							$article_comments = "<div class='posts_contents'>";				
							$article_comments .= $objComments->DrawArticleComments($posts[0]['post_id'], $objCommentsSettings);
							$article_comments .= "</div>";
							
							$article_comments_link = "<img src='images/comments.jpg' alt='' style='float:left;margin-right:3px;' />".$objComments->DrawArticleCommentsLink($posts[0]['post_id']);
						}
					}
				}
			}
			
			// Display post row
			$post_title = "<table width='690px' cellspacing='0' cellpadding='0'><tr><td align='left'>".$posts[0]['post_title']."</td><td align='right'><a href='index.php?admin=posts'>"._BUTTON_BACK."</a></td></td></table>";
			
			$output .= "<div class='post'>
			<table width='97%' align='center'>
			<tr><th align='left'>".draw_title_bar($post_title)."</th><tr>
			<tr>
				<td align='left' style='padding:4px; border-bottom:1px solid #c1c2c3' wrap='wrap'>".$post_text."</td>
			</tr>
			<tr><td nowrap height='1px'></td></tr>
			<tr>
				<td style='color:#575757' colspan='2'>
					<table cellspacing='0' width='100%' border='0'>
					<tr>
						<td width='250px'><i>".str_replace("_AUTHOR_", $posts[0]['author_name'], _POSTED_BY_AUTHOR_AT)." ".format_datetime($posts[0]['posted_at'])."</i></td>
						<td align='left'>".$article_comments_link."</td>
						<td align='right'></td>
					</tr>
					<tr>
						<td colspan='3'>								
							<i>"._TAGS.": ".str_replace(",", " , ", $posts[0]['tags'])."</i>
						</td>
					</tr>
					</table>
				</td>
			</tr>
			<tr><td nowrap height='10px'></td></tr>
			<tr>
				<td>".$article_comments."</td>
			</tr>
			</table>						
			</div><br />";						

			if($comments_status == "open") $output .= "<script>$('#commentsWrapper').show('fast');</script>";
			
			return $output;		
		}else{
			return draw_important_message(_POST_EMPTY);
		}			
	}

	/***
	 * Draw Post
	 *		@param $post_id
	 **/
	function DrawPost($post_id){
		
		global $login;

		$replace_needles = 1;
		$article_comments_link = "";
		$article_comments = "";
		$comments_status = isset($_GET['com_status']) ? $_GET['com_status'] : "";

		$output = "";
		
		$sql = "SELECT
					".DB_PREFIX."posts.id as post_id,
					".DB_PREFIX."posts.category_id,
					".DB_PREFIX."posts.posted_at,
					".DB_PREFIX."posts.post_title,
					".DB_PREFIX."posts.post_text,
					".DB_PREFIX."posts.tags,
					".DB_PREFIX."posts.comments_allowed,
					IF(".DB_PREFIX."categories.category_name IS NULL, 'Other', ".DB_PREFIX."categories.category_name) as category_name,
					IF(".DB_PREFIX."accounts.user_name != '', ".DB_PREFIX."accounts.user_name, 'Admin') as author_name
				FROM ".DB_PREFIX."posts
					LEFT OUTER JOIN ".DB_PREFIX."categories ON ".DB_PREFIX."posts.category_id=".DB_PREFIX."categories.id
					LEFT OUTER JOIN ".DB_PREFIX."accounts  ON ".DB_PREFIX."posts.author_id=".DB_PREFIX."accounts.id
				WHERE
					".DB_PREFIX."posts.is_published = 1 AND 
					".DB_PREFIX."posts.id = '".(int)$post_id."'";
		$posts = database_query($sql, DATA_AND_ROWS, FIRST_ROW_ONLY);
		
		if($posts[1] > 0){
			$post_text = "";			
			if(isset($posts[0]['post_text'])){
				$post_text .= decode_text($posts[0]['post_text'], false);
				
				// include lytebox files
				if(preg_match("/{module:gallery}/i", $post_text)){
					echo "<link rel='stylesheet' href='modules/lytebox/css/lytebox.css' type='text/css' media='screen' />\n";
					echo "<script type='text/javascript' language='javascript' src='modules/lytebox/js/lytebox.js'></script>\n";
					$objGallery = new GalleryAlbums();
					$post_text = preg_replace("/{module:gallery}/i", $objGallery->DrawGallery(), $post_text, 1);
				}
				if(preg_match("/{module:contact}/i", $post_text)){
					if(!@$login->IsLoggedIn()){
						$objContact = new Contact();
						$post_text = preg_replace("/{module:contact}/i", $objContact->DrawContactForm(), $post_text, 1);			
					}else{
						$post_text = preg_replace("/{module:contact}/i", "", $post_text, 1);			
					}
				}
				// draw comments form
				if(Modules::IsModuleInstalled("comments")){
					$objCommentsSettings = new ModulesSettings("comments");
					$comments_allow	= $objCommentsSettings->GetSettings("comments_allow");
					if($comments_allow == "yes" && $posts[0]['comments_allowed']){
						$objComments = new Comments();
						$article_comments = "<div class='posts_contents'>";				
						$article_comments .= $objComments->DrawArticleComments($posts[0]['post_id'], $objCommentsSettings);
						$article_comments .= "</div>";
						
						$article_comments_link = "<img src='images/comments.jpg' alt='' style='float:left;margin-right:3px;' />".$objComments->DrawArticleCommentsLink($posts[0]['post_id']);
					}
				}
			}
			
			// Display post row
			$output .= "<div class='post'>
			<table width='97%' align='center'>
			<tr><th align='left'><a class='post_header_a' href='index.php?page=posts&post_id=".$posts[0]['post_id']."'>".$posts[0]['post_title']."</a></th><tr>
			<tr>
				<td align='left' style='padding:4px; border-bottom:1px solid #c1c2c3' wrap='wrap'>".$post_text."</td>
			</tr>
			<tr><td nowrap height='1px'></td></tr>
			<tr>
				<td style='color:#575757' colspan='2'>
					<table cellspacing='0' width='100%' border='0'>
					<tr>
						<td width='250px'><i>".str_replace("_AUTHOR_", $posts[0]['author_name'], _POSTED_BY_AUTHOR_AT)." ".format_datetime($posts[0]['posted_at'])."</i></td>
						<td align='left'>".$article_comments_link."</td>
						<td align='right'><i>"._CATEGORY_WORD.": <a href='index.php?page=posts&cat_id=".$posts[0]['category_id']."'>".$posts[0]['category_name']."</a></i></td>
					</tr>
					<tr>
						<td colspan='3'>								
							<i>"._TAGS.": ".$this->GetTagLinks($posts[0]['tags'])."</i>
						</td>
					</tr>
					</table>
				</td>
			</tr>
			<tr><td nowrap height='10px'></td></tr>
			<tr>
				<td>".$article_comments."</td>
			</tr>
			</table>						
			</div><br />";						

			$output .= "<div class='adsense_horizontal'>".Adsense::GetHorizontalBanerCode()."</div>";
			
			if($comments_status == "open") $output .= "<script>$('#commentsWrapper').show('fast');</script>";
			
			return $output;		
		}else{
			return draw_important_message(_POST_EMPTY);
		}			
	}
	
	
	private function GetTagLinks($tags){
		$output = "";
		$tags = str_replace(array(", ", " ,", " , "), ",", $tags);
		$tags_parts = explode(",", $tags);
		$tags_parts = array_unique($tags_parts);		
		foreach($tags_parts as $tag){
			if($output != "") $output .= ", ";
			$tag = trim($tag);
			$output .= "<a class='tag' href='index.php?page=posts&tag=".urlencode($tag)."'>".$tag."</a>";	
		}
		return $output;		
	}
	
	private function CleanTags($tags){
		$output = "";
		$tags = str_replace(array(", ", " ,", " , "), ",", $tags);
		$tags_parts = explode(",", $tags);
		$tags_parts = array_unique($tags_parts);		
		foreach($tags_parts as $tag){
			if($output != "") $output .= ",";
			$output .= trim($tag);	
		}
		return $output;
	}	
	
}
?>