<?php

/**
 *	Class Resources
 *  -------------- 
 *  Description : encapsulates Resources operations & properties
 *  Updated	    : 15.11.2009
 *	Written by  : ApPHP
 *	
 **/

class Resources extends MicroGrid {
	
	protected $debug = false;
	
	protected $tableName;
	protected $primaryKey;
	protected $dataSet;

	protected $formActionURL;
	protected $params;
	protected $actions;
	protected $actionIcons;

	protected $allowLanguages;

	protected $VIEW_MODE_SQL;
	protected $EDIT_MODE_SQL;
	protected $DETAILS_MODE_SQL;
	protected $WHERE_CLAUSE;
	protected $ORDER_CLAUSE;
	protected $LIMIT_CLAUSE;

	protected $arrViewModeFields;	
	protected $arrAddModeFields;	
	protected $arrEditModeFields;
	protected $arrDetailsModeFields;	
	
	protected $isPagingAllowed;
	protected $postSize;

	protected $isSortingAllowed;

	protected $isFilteringAllowed;
	protected $arrFilteringFields;

	public $error;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct() {
		
		$this->SetRunningTime();

		$this->params = array();		
		if(isset($_POST['link_text']))    $this->params['link_text']    = $_POST['link_text'];
		if(isset($_POST['link_href'])) 	  $this->params['link_href']    = $_POST['link_href'];
		if(isset($_POST['link_target']))  $this->params['link_target']  = $_POST['link_target'];
	
		$this->primaryKey 	= "id";
		$this->tableName 	= DB_PREFIX."resources";
		$this->dataSet 		= array();
		$this->error 		= "";
		$this->formActionURL = "index.php?admin=mod_resources_management";
		$this->actions      = array("add"=>true, "edit"=>true, "details"=>true, "delete"=>true);

		$this->WHERE_CLAUSE = "";		
		$this->ORDER_CLAUSE = "ORDER BY id DESC";

		$this->allowLanguages = false;
	
		$this->isAlterColorsAllowed = true;

		$this->isPagingAllowed = true;
		$this->pageSize = 20;

		$this->isSortingAllowed = true;

		$this->isFilteringAllowed = false;		
		
		$objResourcesSettings = new ModulesSettings("resources");
		$default_target	= $objResourcesSettings->GetSettings("resources_default_target");
		// define filtering fields
		///$this->arrFilteringFields = array(
		///	"parameter1" => array("title"=>"",  "type"=>"text", "sign"=>"=|like%|%like|%like%", "width"=>"80px"),
		///	"parameter2"  => array("title"=>"",  "type"=>"text", "sign"=>"=|like%|%like|%like%", "width"=>"80px"),
		///);
		
		//---------------------------------------------------------------------- 
		// VIEW MODE
		//---------------------------------------------------------------------- 
		$this->VIEW_MODE_SQL = "SELECT ".$this->primaryKey.",
									link_text,
									link_href,
									link_target
								FROM ".$this->tableName."";		
		// define view mode fields
		$this->arrViewModeFields = array(
			"link_text"    => array("title"=>_HEADER,	"type"=>"label", "align"=>"left", "width"=>""),
			"link_href"    => array("title"=>_LINK,		"type"=>"label", "align"=>"left", "width"=>""),
			"link_target"  => array("title"=>_TARGET,	"type"=>"label", "align"=>"center", "width"=>"100px"),
		);
		
		//---------------------------------------------------------------------- 
		// ADD MODE
		//---------------------------------------------------------------------- 
		// define add mode fields
		$this->arrAddModeFields = array(
			"link_text"    => array("title"=>_HEADER,	"type"=>"textbox",  "width"=>"210px", "required"=>true, "readonly"=>false, "maxlength"=>"", "default"=>"", "validation_type"=>"text"),
			"link_href"    => array("title"=>_LINK." (http://)", "type"=>"textbox",  "width"=>"310px", "required"=>true, "readonly"=>false, "maxlength"=>"", "default"=>"", "validation_type"=>"text"),
				"link_target"  => array("title"=>_TARGET."<br />(_blank, _parent, _self ...)",	"type"=>"textbox",  "width"=>"90px", "required"=>false, "readonly"=>false, "maxlength"=>"20", "default"=>$default_target, "validation_type"=>"text"),
		);

		//---------------------------------------------------------------------- 
		// EDIT MODE
		//---------------------------------------------------------------------- 
		$this->EDIT_MODE_SQL = "SELECT
								".$this->tableName.".".$this->primaryKey.",
								".$this->tableName.".link_text,
								".$this->tableName.".link_href,
								".$this->tableName.".link_target
							FROM ".$this->tableName."
							WHERE ".$this->tableName.".".$this->primaryKey." = _RID_";		
		// define edit mode fields
		$this->arrEditModeFields = array(
			"link_text"    => array("title"=>_HEADER, "type"=>"textbox",  "width"=>"210px", "required"=>true, "readonly"=>false, "maxlength"=>"", "default"=>"", "validation_type"=>"text"),
			"link_href"    => array("title"=>_LINK." (http://)",   "type"=>"textbox",  "width"=>"310px", "required"=>true, "readonly"=>false, "maxlength"=>"", "default"=>"", "validation_type"=>"text"),
			"link_target"  => array("title"=>_TARGET."<br />(_blank, _parent, _self ...)",	"type"=>"textbox",  "width"=>"90px", "required"=>false, "readonly"=>false, "maxlength"=>"20", "default"=>"", "validation_type"=>"text"),
		);

		//---------------------------------------------------------------------- 
		// DETAILS MODE
		//----------------------------------------------------------------------
		$this->DETAILS_MODE_SQL = $this->EDIT_MODE_SQL;
		$this->arrDetailsModeFields = array(
			"link_text"     => array("title"=>_HEADER, "type"=>"label"),
			"link_href"     => array("title"=>_LINK,   "type"=>"label"),
			"link_href"     => array("title"=>_TARGET, "type"=>"label"),
		);

	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct()
	{
		// echo 'this object has been destroyed';
    }


	/***
	 *	Get all DataSet array
	 *
	 *  static public function GetAll() { }
	 *	
	 **/
	
	/***
	 *	Add record
	 *
	 *	function AddRecord($params = array()) { 	}
	 *	
	 **/

	/***
	 *	Delete record
	 *
	 *	function DeleteRecord($rid = "") { 	}
	 *	
	 **/
	
	
	function DrawResourcesBlock()
	{
		global $objSettings;
	    $text_align_left = ($objSettings->GetDirection() == "ltr") ? "text-align: left;" : "text-align: right; padding-right:15px;";
		$text_align_right = ($objSettings->GetDirection() == "ltr") ? "text-align: right; padding-right:15px;" : "text-align: left;";

		$objResourcesSettings = new ModulesSettings("resources");
		$resources_header_length = $objResourcesSettings->GetSettings("resources_header_len");
		$resources_count 		 = $objResourcesSettings->GetSettings("resources_count");
		$resources_target 		 = $objResourcesSettings->GetSettings("resources_target"); 

		$this->WHERE_CLAUSE = "";		
		$all_resources = $this->GetAll($this->ORDER_CLAUSE);
		//print_r($all_resources);
		
		echo "<table class='moduletable' width='100%'>";
		echo "<tr><th valign='top' style='".$text_align_left."'>".strtoupper(_RESOURCES)."</th></tr>";
		for($resources_ind = 0; $resources_ind < $all_resources[1]; $resources_ind++)
		{
			if($resources_ind+1 > $resources_count) break;              // Show first X resources
			$resources_str    = $all_resources[0][$resources_ind]['link_text'];  // Display Y first chars
			$resources_href   = $all_resources[0][$resources_ind]['link_href'];  //
			$resources_target = $all_resources[0][$resources_ind]['link_target'];  // 
			
			$resources_str = (strlen($resources_str) > $resources_header_length) ? substr($all_resources[0][$resources_ind]['link_text'],0,$resources_header_length)."..." : $resources_str;
			echo "<tr><td valign='top' style='".$text_align_left."'><a class='menu' href='".$resources_href."' target='".$resources_target."'>".$resources_str."</a></td></tr>";
		}
		if($resources_ind == 0){
			echo "<tr><td align='center'>"._NO_RESOURCES."</td></tr>";
		}
		echo "</table>";
	}
	
	
	/***
	 *	Get resources
	 *
	 *		@param $resources_id
	 *	
	 **/
	function GetResources($resources_id = "0")
	{
		$sql = $this->VIEW_MODE_SQL." WHERE ".$this->primaryKey." = ".$resources_id." ".$this->ORDER_CLAUSE;
		$resources = $this->GetRecord($sql);
		return $resources;		
	}

}
?>