<?php

/***
 *	Class Settings
 *  ----------------- 
 *  Description : encapsulates site's settings and properties
 *  Updated	    : 25.04.2010
 *	Written by  : ApPHP
 *	
 **/

class Settings {

	public $error;
	
	private $res;
	private $template;
	private $header_text;
	private $footer_text;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct(){
		$this->error = "";
		$this->template = "";
		$this->header_text = "";
		$this->footer_text = "";
		$this->test_direction = "";
		
		$this->GetSettings();
	}

	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct(){
		// echo 'this object has been destroyed';
    }

	/***
	 *	Refresh settings
	 **/
	public function Refresh(){
		$this->GetSettings();
	}

	/***
	 *	Refresh settings
	 **/
	private function GetSettings(){
		$sql = "SELECT * FROM ".DB_PREFIX."settings";
		$this->res = database_query($sql, DATA_ONLY, FIRST_ROW_ONLY);
		
		if($this->res){
			$this->header_text    = $this->res['header_text'];
			$this->footer_text    = $this->res['footer_text'];
			$this->template       = $this->res['template'];
			$this->text_direction = $this->res['text_direction'];
		}
	}

	/***
	 *	Set text direction
	 *		@param $text_direction - ltr|rtl
	 **/
	public function SetDirection($text_direction = "ltr"){
		// check if this is a DEMO
		if(strtolower(SITE_MODE) == "demo"){ $this->error = _OPERATION_BLOCKED; return false; }

		$sql = "UPDATE ".DB_PREFIX."settings SET text_direction = '".$text_direction."'";
		if (database_void_query($sql)){
			$this->template = $template;
			return true;
		} else {
			$this->error = _TRY_LATER;
			return false;
		}							
	}
	
	/***
	 *	Get current text direction
	 *		
	 **/
	public function GetDirection(){
		return $this->text_direction;
	}

	/***
	 *	Get template name
	 *		
	 **/
	public function GetTemplate(){
		return decode_text($this->template);
	}
	
	/***
	 *	Get parameter value by name
	 *		@param $field_name	
	 **/
	public function GetParameter($field_name = ""){
		if(isset($this->res[$field_name])){
			return decode_text($this->res[$field_name]);	
		}else{
			return "";
		}
	}

	/***
	 *	Update fields
	 *		@param $params - pairs: field - value 
	 **/
	public function UpdateFields($params = array()){
		
		// check if this is a DEMO
		if(strtolower(SITE_MODE) == "demo"){ $this->error = _OPERATION_BLOCKED; return false; }
		
		if(count($params) > 0){
			// prepare UPDATE statement
			$sql = "UPDATE ".DB_PREFIX."settings SET ";
			$count = 0;
			foreach($params as $key => $val){
				if($count++ > 0) $sql .= ", ";
				$sql .= $key." = '".$val."'";				
			}
			if (database_void_query($sql)){
				return true;
			} else {
				$this->error = _TRY_LATER;
				return false;
			}				
		}else{
			return "";						
		}
	}	

	/***
	 *	Set site template
	 *		@param $template - template name
	 **/
	public function SetTemplate($template = ""){

		// check if this is a DEMO
		if(strtolower(SITE_MODE) == "demo"){ $this->error = _OPERATION_BLOCKED; return false; }
		
		$sql = "UPDATE ".DB_PREFIX."settings SET template = '".encode_text($template)."'";
		if (database_void_query($sql)){
			$this->template = $template;
			return true;
		} else {
			$this->error = _TRY_LATER;
			return false;
		}							
	}
	
	/***
	 *	Set text direction
	 *		@param $text_direction - ltr|rtl
	 **/
	public function GetHeader(){
		return decode_text($this->header_text);
	}

	/***
	 *	Set site header
	 *		@param $header_text
	 **/
	public function SetHeader($header_text = ""){

		// check if this is a DEMO
		if(strtolower(SITE_MODE) == "demo"){ $this->error = _OPERATION_BLOCKED; return false; }

		$sql = "UPDATE ".DB_PREFIX."settings SET header_text = '".encode_text($header_text)."'";
		if(database_void_query($sql)){
			$this->header_text = $header_text;
			return true;
		} else {
			$this->error = _TRY_LATER;
			return false;
		}							
	}
	
	/***
	 *	Draw site's footer
	 *		
	 **/
	public function DrawFooter(){
		echo $this->GetDataTagged($this->footer_text);
	}

	/***
	 *	Get site's footer text
	 *		
	 **/
	public function GetFooter(){
		return decode_text($this->footer_text);
	}

	/***
	 *	Set site's footer text
	 *		@param $footer_text
	 **/
	public function SetFooter($footer_text = ""){

		// check if this is a DEMO
		if(strtolower(SITE_MODE) == "demo"){ $this->error = _OPERATION_BLOCKED; return false; }

		$sql = "UPDATE ".DB_PREFIX."settings SET footer_text = '".encode_text($footer_text)."'";
		if (database_void_query($sql)){
			$this->footer_text = $footer_text;
			return true;
		} else {
			$this->error = _TRY_LATER;
			return false;
		}							
	}

	/***
	 *	Get tagged string 
	 *		@param $string
	 **/
	function GetDataTagged($string){
		$string = str_replace("&lt;", "<", $string); 
		$string = str_replace("&gt;", ">", $string); 
		return $string;		
	}
	
}
?>